import copy

from geoformat.conf.error_messages import (
    field_missing,
    field_exists,
    field_type_not_valid,
    field_width_not_valid,
    field_precision_not_valid,
    variable_must_be_int,
    field_name_still_indexing,
    field_name_not_indexing,
)
from tests.data.index import geolayer_fr_dept_population_CODE_DEPT_hash_index
from geoformat.conf.fields_variable import field_type_set
from geoformat.manipulation.metadata_manipulation import (
    drop_field_in_metadata,
    drop_field_that_not_exists_in_metadata,
    reorder_metadata_field_index_after_field_drop,
    rename_field_in_metadata,
    create_field_in_metadata,
    check_if_field_exists_in_metadata,
    add_attributes_index_in_metadata,
    check_attributes_index_in_metadata,
    delete_attributes_index_in_metadata,
)
from tests.utils.tests_utils import test_function

from tests.data.metadata import (
    metadata_attributes_to_force_in_str,
    metadata_attributes_only,
    metadata_fr_dept_geometry_only,
    metadata_fr_dept_population,
    metadata_geometry_only_all_geometries_type,
)

reorder_metadata_field_index_after_field_drop_parameters = {
    0: {
        "fields_metadata": {"NOM_DEPT": {"type": "String", "width": 10, "index": 1}},
        "reorder_field_index": 0,
        "return_value": {"NOM_DEPT": {"type": "String", "width": 10, "index": 0}},
    },
    1: {
        "fields_metadata": {"CODE_DEPT": {"type": "String", "width": 2, "index": 0}},
        "reorder_field_index": 1,
        "return_value": {"CODE_DEPT": {"type": "String", "width": 2, "index": 0}},
    },
    2: {
        "fields_metadata": {
            "field_integer": {"type": "Integer", "index": 0},
            "field_integer_list": {"type": "IntegerList", "index": 1},
            "field_real": {"type": "Real", "width": 6, "precision": 0, "index": 2},
            "field_real_list": {
                "type": "RealList",
                "width": 10,
                "precision": 5,
                "index": 3,
            },
            "field_date": {"type": "Date", "index": 6},
            "field_time": {"type": "Time", "index": 7},
            "field_datetime": {"type": "DateTime", "index": 8},
            "field_binary": {"type": "Binary", "index": 9},
            "field_boolean": {"type": "Boolean", "index": 10},
        },
        "reorder_field_index": [4, 5],
        "return_value": {
            "field_integer": {"type": "Integer", "index": 0},
            "field_integer_list": {"type": "IntegerList", "index": 1},
            "field_real": {"type": "Real", "width": 6, "precision": 0, "index": 2},
            "field_real_list": {
                "type": "RealList",
                "width": 10,
                "precision": 5,
                "index": 3,
            },
            "field_date": {"type": "Date", "index": 4},
            "field_time": {"type": "Time", "index": 5},
            "field_datetime": {"type": "DateTime", "index": 6},
            "field_binary": {"type": "Binary", "index": 7},
            "field_boolean": {"type": "Boolean", "index": 8},
        },
    },
    3: {
        "fields_metadata": {
            "field_real": {"type": "Real", "width": 6, "precision": 0, "index": 2},
            "field_real_list": {
                "type": "RealList",
                "width": 10,
                "precision": 5,
                "index": 3,
            },
            "field_string": {"type": "String", "width": 5, "index": 4},
            "field_string_list": {"type": "StringList", "width": 8, "index": 5},
            "field_date": {"type": "Date", "index": 6},
            "field_time": {"type": "Time", "index": 7},
            "field_datetime": {"type": "DateTime", "index": 8},
            "field_binary": {"type": "Binary", "index": 9},
            "field_boolean": {"type": "Boolean", "index": 10},
        },
        "reorder_field_index": [0, 1],
        "return_value": {
            "field_real": {"type": "Real", "width": 6, "precision": 0, "index": 0},
            "field_real_list": {
                "type": "RealList",
                "width": 10,
                "precision": 5,
                "index": 1,
            },
            "field_string": {"type": "String", "width": 5, "index": 2},
            "field_string_list": {"type": "StringList", "width": 8, "index": 3},
            "field_date": {"type": "Date", "index": 4},
            "field_time": {"type": "Time", "index": 5},
            "field_datetime": {"type": "DateTime", "index": 6},
            "field_binary": {"type": "Binary", "index": 7},
            "field_boolean": {"type": "Boolean", "index": 8},
        },
    },
    4: {
        "fields_metadata": {
            "field_integer": {"type": "Integer", "index": 0},
            "field_integer_list": {"type": "IntegerList", "index": 1},
            "field_real": {"type": "Real", "width": 6, "precision": 0, "index": 2},
            "field_real_list": {
                "type": "RealList",
                "width": 10,
                "precision": 5,
                "index": 3,
            },
            "field_string": {"type": "String", "width": 5, "index": 4},
            "field_string_list": {"type": "StringList", "width": 8, "index": 5},
            "field_date": {"type": "Date", "index": 6},
            "field_time": {"type": "Time", "index": 7},
        },
        "reorder_field_index": [8, 9, 10],
        "return_value": {
            "field_integer": {"type": "Integer", "index": 0},
            "field_integer_list": {"type": "IntegerList", "index": 1},
            "field_real": {"type": "Real", "width": 6, "precision": 0, "index": 2},
            "field_real_list": {
                "type": "RealList",
                "width": 10,
                "precision": 5,
                "index": 3,
            },
            "field_string": {"type": "String", "width": 5, "index": 4},
            "field_string_list": {"type": "StringList", "width": 8, "index": 5},
            "field_date": {"type": "Date", "index": 6},
            "field_time": {"type": "Time", "index": 7},
        },
    },
}

drop_field_in_metadata_parameters = {
    0: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name_or_field_name_list": "field_integer",
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer_list": {"type": "String", "width": 13, "index": 0},
                "field_real": {"type": "String", "width": 7, "index": 1},
                "field_real_list": {"type": "String", "width": 19, "index": 2},
                "field_string": {"type": "String", "width": 26, "index": 3},
                "field_date": {"type": "Date", "index": 4},
                "field_time": {"type": "String", "width": 15, "index": 5},
                "field_binary": {"type": "String", "width": 201, "index": 6},
                "field_boolean": {"type": "String", "width": 5, "index": 7},
                "field_string_list": {"type": "String", "width": 16, "index": 8},
                "field_datetime": {"type": "DateTime", "index": 9},
            },
        },
    },
    1: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name_or_field_name_list": "field_datetime",
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer": {
                    "type": "Real",
                    "width": 9,
                    "precision": 5,
                    "index": 0,
                },
                "field_integer_list": {"type": "String", "width": 13, "index": 1},
                "field_real": {"type": "String", "width": 7, "index": 2},
                "field_real_list": {"type": "String", "width": 19, "index": 3},
                "field_string": {"type": "String", "width": 26, "index": 4},
                "field_date": {"type": "Date", "index": 5},
                "field_time": {"type": "String", "width": 15, "index": 6},
                "field_binary": {"type": "String", "width": 201, "index": 7},
                "field_boolean": {"type": "String", "width": 5, "index": 8},
                "field_string_list": {"type": "String", "width": 16, "index": 9},
            },
        },
    },
    2: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name_or_field_name_list": "field_string",
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer": {
                    "type": "Real",
                    "width": 9,
                    "precision": 5,
                    "index": 0,
                },
                "field_integer_list": {"type": "String", "width": 13, "index": 1},
                "field_real": {"type": "String", "width": 7, "index": 2},
                "field_real_list": {"type": "String", "width": 19, "index": 3},
                "field_date": {"type": "Date", "index": 4},
                "field_time": {"type": "String", "width": 15, "index": 5},
                "field_binary": {"type": "String", "width": 201, "index": 6},
                "field_boolean": {"type": "String", "width": 5, "index": 7},
                "field_string_list": {"type": "String", "width": 16, "index": 8},
                "field_datetime": {"type": "DateTime", "index": 9},
            },
        },
    },
    3: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name_or_field_name_list": [
            "field_datetime",
            "field_string",
            "field_integer",
        ],
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer_list": {"type": "String", "width": 13, "index": 0},
                "field_real": {"type": "String", "width": 7, "index": 1},
                "field_real_list": {"type": "String", "width": 19, "index": 2},
                "field_date": {"type": "Date", "index": 3},
                "field_time": {"type": "String", "width": 15, "index": 4},
                "field_binary": {"type": "String", "width": 201, "index": 5},
                "field_boolean": {"type": "String", "width": 5, "index": 6},
                "field_string_list": {"type": "String", "width": 16, "index": 7},
            },
        },
    },
    4: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name_or_field_name_list": [
            "field_string_list",
            "field_binary",
            "field_time",
            "field_boolean",
            "field_integer_list",
            "field_datetime",
            "field_real",
            "field_string",
            "field_integer",
            "field_real_list",
            "field_date",
        ],
        "return_value": {"name": "attributes_to_force_only_forced"},
    },
}

drop_field_that_not_exists_in_metadata_parameters = {
    0: {
        "metadata": copy.deepcopy(metadata_attributes_to_force_in_str),
        "not_deleting_field_name_or_field_name_list": [
            "field_integer_list",
            "field_real",
            "field_real_list",
            "field_string",
            "field_date",
            "field_time",
            "field_binary",
            "field_boolean",
            "field_string_list",
            "field_datetime",
        ],
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer_list": {"type": "String", "width": 13, "index": 0},
                "field_real": {"type": "String", "width": 7, "index": 1},
                "field_real_list": {"type": "String", "width": 19, "index": 2},
                "field_string": {"type": "String", "width": 26, "index": 3},
                "field_date": {"type": "Date", "index": 4},
                "field_time": {"type": "String", "width": 15, "index": 5},
                "field_binary": {"type": "String", "width": 201, "index": 6},
                "field_boolean": {"type": "String", "width": 5, "index": 7},
                "field_string_list": {"type": "String", "width": 16, "index": 8},
                "field_datetime": {"type": "DateTime", "index": 9},
            },
        },
    },
    1: {
        "metadata": copy.deepcopy(metadata_attributes_to_force_in_str),
        "not_deleting_field_name_or_field_name_list": [
            "field_integer",
            "field_integer_list",
            "field_real",
            "field_real_list",
            "field_string",
            "field_date",
            "field_time",
            "field_binary",
            "field_boolean",
            "field_string_list",
        ],
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer": {
                    "type": "Real",
                    "width": 9,
                    "precision": 5,
                    "index": 0,
                },
                "field_integer_list": {"type": "String", "width": 13, "index": 1},
                "field_real": {"type": "String", "width": 7, "index": 2},
                "field_real_list": {"type": "String", "width": 19, "index": 3},
                "field_string": {"type": "String", "width": 26, "index": 4},
                "field_date": {"type": "Date", "index": 5},
                "field_time": {"type": "String", "width": 15, "index": 6},
                "field_binary": {"type": "String", "width": 201, "index": 7},
                "field_boolean": {"type": "String", "width": 5, "index": 8},
                "field_string_list": {"type": "String", "width": 16, "index": 9},
            },
        },
    },
    2: {
        "metadata": copy.deepcopy(metadata_attributes_to_force_in_str),
        "not_deleting_field_name_or_field_name_list": [
            "field_integer",
            "field_integer_list",
            "field_real",
            "field_real_list",
            "field_date",
            "field_time",
            "field_binary",
            "field_boolean",
            "field_string_list",
            "field_datetime",
        ],
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer": {
                    "type": "Real",
                    "width": 9,
                    "precision": 5,
                    "index": 0,
                },
                "field_integer_list": {"type": "String", "width": 13, "index": 1},
                "field_real": {"type": "String", "width": 7, "index": 2},
                "field_real_list": {"type": "String", "width": 19, "index": 3},
                "field_date": {"type": "Date", "index": 4},
                "field_time": {"type": "String", "width": 15, "index": 5},
                "field_binary": {"type": "String", "width": 201, "index": 6},
                "field_boolean": {"type": "String", "width": 5, "index": 7},
                "field_string_list": {"type": "String", "width": 16, "index": 8},
                "field_datetime": {"type": "DateTime", "index": 9},
            },
        },
    },
    3: {
        "metadata": metadata_attributes_to_force_in_str,
        "not_deleting_field_name_or_field_name_list": [
            "field_integer_list",
            "field_real",
            "field_real_list",
            "field_date",
            "field_time",
            "field_binary",
            "field_boolean",
            "field_string_list",
        ],
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer_list": {"type": "String", "width": 13, "index": 0},
                "field_real": {"type": "String", "width": 7, "index": 1},
                "field_real_list": {"type": "String", "width": 19, "index": 2},
                "field_date": {"type": "Date", "index": 3},
                "field_time": {"type": "String", "width": 15, "index": 4},
                "field_binary": {"type": "String", "width": 201, "index": 5},
                "field_boolean": {"type": "String", "width": 5, "index": 6},
                "field_string_list": {"type": "String", "width": 16, "index": 7},
            },
        },
    },
    4: {
        "metadata": copy.deepcopy(metadata_attributes_to_force_in_str),
        "not_deleting_field_name_or_field_name_list": [],
        "return_value": {"name": "attributes_to_force_only_forced"},
    },
    5: {
        "metadata": {'name': 'geolayer_dpt_2_with_geom', 'fields': {'CODE_DEPT': {'type': 'String', 'width': 2, 'index': 0}, 'NOM_DEPT': {'type': 'String', 'width': 10, 'index': 1}}, 'geometry_ref': {'type': {'Polygon'}, 'extent': (598361.0, 6861428.0, 790134.0, 6997000.0)}},
        "not_deleting_field_name_or_field_name_list": ['CODE_DEPT', 'NOM_DEPT'],
        "return_value": {'name': 'geolayer_dpt_2_with_geom', 'fields': {'CODE_DEPT': {'type': 'String', 'width': 2, 'index': 0}, 'NOM_DEPT': {'type': 'String', 'width': 10, 'index': 1}}, 'geometry_ref': {'type': {'Polygon'}, 'extent': (598361.0, 6861428.0, 790134.0, 6997000.0)}},
    },

}

rename_field_in_metadata_parameters = {
    0: {
        "metadata": metadata_attributes_only,
        "old_field_name": "field_string",
        "new_field_name": "string",
        "return_value": {
            "name": "attributes_only",
            "fields": {
                "field_integer": {"type": "Integer", "index": 0},
                "field_integer_list": {"type": "IntegerList", "index": 1},
                "field_real": {"type": "Real", "width": 9, "precision": 5, "index": 2},
                "field_real_list": {
                    "type": "RealList",
                    "width": 10,
                    "precision": 5,
                    "index": 3,
                },
                "field_string_list": {"type": "StringList", "width": 8, "index": 5},
                "field_date": {"type": "Date", "index": 6},
                "field_time": {"type": "Time", "index": 7},
                "field_datetime": {"type": "DateTime", "index": 8},
                "field_binary": {"type": "Binary", "index": 9},
                "field_boolean": {"type": "Boolean", "index": 10},
                "string": {"type": "String", "width": 5, "index": 4},
            },
        },
    },
    1: {
        "metadata": metadata_attributes_only,
        "old_field_name": "field_foo",
        "new_field_name": "string",
        "return_value": field_missing.format(field_name="field_foo"),
    },
    2: {
        "metadata": metadata_attributes_only,
        "old_field_name": "field_binary",
        "new_field_name": "field_string",
        "return_value": field_exists.format(field_name="field_string"),
    },
}

create_field_in_metadata_parameters = {
    0: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name": "field_string",
        "field_type": "int",
        "field_width": None,
        "field_precision": None,
        "return_value": field_exists.format(field_name="field_string"),
    },
    1: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name": "foo",
        "field_type": "int",
        "field_width": None,
        "field_precision": None,
        "return_value": field_type_not_valid.format(
            field_type="int", field_type_list=str(sorted(list(field_type_set)))
        ),
    },
    2: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name": "foo",
        "field_type": "Real",
        "field_width": None,
        "field_precision": None,
        "return_value": field_width_not_valid.format(field_name="foo"),
    },
    3: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name": "foo",
        "field_type": "Real",
        "field_width": "bar",
        "field_precision": None,
        "return_value": variable_must_be_int.format(variable_name="field_width"),
    },
    4: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name": "foo",
        "field_type": "Real",
        "field_width": 5,
        "field_precision": None,
        "return_value": field_precision_not_valid.format(field_name="foo"),
    },
    5: {
        "metadata": metadata_attributes_to_force_in_str,
        "field_name": "foo",
        "field_type": "Real",
        "field_width": 5,
        "field_precision": "deux",
        "return_value": variable_must_be_int.format(variable_name="field_precision"),
    },
    6: {
        "metadata": copy.deepcopy(metadata_attributes_to_force_in_str),
        "field_name": "foo",
        "field_type": "Integer",
        "field_width": None,
        "field_precision": None,
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer": {
                    "type": "Real",
                    "width": 9,
                    "precision": 5,
                    "index": 0,
                },
                "field_integer_list": {"type": "String", "width": 13, "index": 1},
                "field_real": {"type": "String", "width": 7, "index": 2},
                "field_real_list": {"type": "String", "width": 19, "index": 3},
                "field_string": {"type": "String", "width": 26, "index": 4},
                "field_date": {"type": "Date", "index": 5},
                "field_time": {"type": "String", "width": 15, "index": 6},
                "field_binary": {"type": "String", "width": 201, "index": 7},
                "field_boolean": {"type": "String", "width": 5, "index": 8},
                "field_string_list": {"type": "String", "width": 16, "index": 9},
                "field_datetime": {"type": "DateTime", "index": 10},
                "foo": {"type": "Integer", "index": 11},
            },
        },
    },
    7: {
        "metadata": copy.deepcopy(metadata_attributes_to_force_in_str),
        "field_name": "foo",
        "field_type": "Integer",
        "field_width": 10,
        "field_precision": 2,
        "return_value": {
            "name": "attributes_to_force_only_forced",
            "fields": {
                "field_integer": {
                    "type": "Real",
                    "width": 9,
                    "precision": 5,
                    "index": 0,
                },
                "field_integer_list": {"type": "String", "width": 13, "index": 1},
                "field_real": {"type": "String", "width": 7, "index": 2},
                "field_real_list": {"type": "String", "width": 19, "index": 3},
                "field_string": {"type": "String", "width": 26, "index": 4},
                "field_date": {"type": "Date", "index": 5},
                "field_time": {"type": "String", "width": 15, "index": 6},
                "field_binary": {"type": "String", "width": 201, "index": 7},
                "field_boolean": {"type": "String", "width": 5, "index": 8},
                "field_string_list": {"type": "String", "width": 16, "index": 9},
                "field_datetime": {"type": "DateTime", "index": 10},
                "foo": {"type": "Integer", "index": 11},
            },
        },
    },
    8: {
        "metadata": copy.deepcopy(metadata_fr_dept_geometry_only),
        "field_name": "foo",
        "field_type": "Integer",
        "field_width": 10,
        "field_precision": 2,
        "return_value": {
            "name": "FRANCE_DPT_GENERALIZE_LAMB93_ROUND_GEOMETRY_ONLY",
            "geometry_ref": {
                "type": {"Polygon", "MultiPolygon"},
                "extent": (124277.0, 6050136.0, 1242213.0, 7110430.0),
                "crs": 2154,
            },
            "fields": {"foo": {"type": "Integer", "index": 0}},
        },
    },
}

add_attributes_index_in_metadata_parameters = {
    0: {
        "metadata": copy.deepcopy(metadata_fr_dept_population),
        "field_name": "CODE_DEPT",
        "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
        "return_value": {
            "name": "dept_population",
            "fields": {
                "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                "INSEE_REG": {"type": "String", "width": 2, "index": 1},
                "POPULATION": {"type": "Integer", "index": 2},
                "AREA": {"type": "Real", "width": 7, "precision": 2, "index": 3},
                "DENSITY": {"type": "Real", "width": 7, "precision": 2, "index": 4},
            },
            "index": {
                "attributes": {
                    "CODE_DEPT": {
                        "metadata": {"type": "hashtable"},
                        "index": {
                            "32": [0],
                            "47": [1],
                            "38": [2],
                            "62": [3],
                            "08": [4],
                            "10": [5],
                            "42": [6],
                            "06": [7],
                            "31": [8],
                            "71": [9],
                            "53": [10],
                            "78": [11],
                            "50": [12],
                            "16": [13],
                            "25": [14],
                            "55": [15],
                            "33": [16],
                            "14": [17],
                            "88": [18],
                            "18": [19],
                            "07": [20],
                            "02": [21],
                            "64": [22],
                            "41": [23],
                            "57": [24],
                            "86": [25],
                            "24": [26],
                            "39": [27],
                            "82": [28],
                            "49": [29],
                            "69": [30],
                            "12": [31],
                            "23": [32],
                            "45": [33],
                            "70": [34],
                            "63": [35],
                            "81": [36],
                            "27": [37],
                            "76": [38],
                            "52": [39],
                            "30": [40],
                            "67": [41],
                            "11": [42],
                            "77": [43],
                            "43": [44],
                            "51": [45],
                            "80": [46],
                            "46": [47],
                            "65": [48],
                            "04": [49],
                            "72": [50],
                            "56": [51],
                            "2A": [52],
                            "28": [53],
                            "54": [54],
                            "01": [55],
                            "19": [56],
                            "09": [57],
                            "68": [58],
                            "59": [59],
                            "90": [60],
                            "44": [61],
                            "89": [62],
                            "35": [63],
                            "40": [64],
                            "29": [65],
                            "74": [66],
                            "60": [67],
                            "95": [68],
                            "58": [69],
                            "61": [70],
                            "91": [71],
                            "21": [72],
                            "22": [73],
                            "03": [74],
                            "17": [75],
                            "15": [76],
                            "34": [77],
                            "26": [78],
                            "66": [79],
                            "73": [80],
                            "37": [81],
                            "05": [82],
                            "79": [83],
                            "84": [84],
                            "36": [85],
                            "2B": [86],
                            "87": [87],
                            "85": [88],
                            "83": [89],
                            "94": [90],
                            "92": [91],
                            "48": [92],
                            "13": [93],
                            "93": [94],
                            "75": [95],
                        },
                    }
                }
            },
        },
    },
    1: {
        "metadata": {
            "name": "dept_population",
            "fields": {
                "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                "INSEE_REG": {"type": "String", "width": 2, "index": 1},
                "POPULATION": {"type": "Integer", "index": 2},
                "AREA": {"type": "Real", "width": 7, "precision": 2, "index": 3},
                "DENSITY": {"type": "Real", "width": 7, "precision": 2, "index": 4},
            },
            "index": {
                "attributes": {
                    "CODE_DEPT": {
                        "type": "hashtable",
                        "index": {
                            "32": [0],
                            "47": [1],
                            "38": [2],
                            "62": [3],
                            "08": [4],
                            "10": [5],
                            "42": [6],
                            "06": [7],
                            "31": [8],
                            "71": [9],
                            "53": [10],
                            "78": [11],
                            "50": [12],
                            "16": [13],
                            "25": [14],
                            "55": [15],
                            "33": [16],
                            "14": [17],
                            "88": [18],
                            "18": [19],
                            "07": [20],
                            "02": [21],
                            "64": [22],
                            "41": [23],
                            "57": [24],
                            "86": [25],
                            "24": [26],
                            "39": [27],
                            "82": [28],
                            "49": [29],
                            "69": [30],
                            "12": [31],
                            "23": [32],
                            "45": [33],
                            "70": [34],
                            "63": [35],
                            "81": [36],
                            "27": [37],
                            "76": [38],
                            "52": [39],
                            "30": [40],
                            "67": [41],
                            "11": [42],
                            "77": [43],
                            "43": [44],
                            "51": [45],
                            "80": [46],
                            "46": [47],
                            "65": [48],
                            "04": [49],
                            "72": [50],
                            "56": [51],
                            "2A": [52],
                            "28": [53],
                            "54": [54],
                            "01": [55],
                            "19": [56],
                            "09": [57],
                            "68": [58],
                            "59": [59],
                            "90": [60],
                            "44": [61],
                            "89": [62],
                            "35": [63],
                            "40": [64],
                            "29": [65],
                            "74": [66],
                            "60": [67],
                            "95": [68],
                            "58": [69],
                            "61": [70],
                            "91": [71],
                            "21": [72],
                            "22": [73],
                            "03": [74],
                            "17": [75],
                            "15": [76],
                            "34": [77],
                            "26": [78],
                            "66": [79],
                            "73": [80],
                            "37": [81],
                            "05": [82],
                            "79": [83],
                            "84": [84],
                            "36": [85],
                            "2B": [86],
                            "87": [87],
                            "85": [88],
                            "83": [89],
                            "94": [90],
                            "92": [91],
                            "48": [92],
                            "13": [93],
                            "93": [94],
                            "75": [95],
                        },
                    }
                }
            },
        },
        "field_name": "CODE_DEPT",
        "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
        "return_value": field_name_still_indexing.format(field_name="CODE_DEPT"),
    },
    2: {
        "metadata": metadata_fr_dept_population,
        "field_name": "CODE",
        "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
        "return_value": field_missing.format(field_name="CODE"),
    },
    3: {
        "metadata": metadata_geometry_only_all_geometries_type,
        "field_name": "CODE_DEPT",
        "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
        "return_value": field_missing.format(field_name="CODE_DEPT"),
    },
}

check_if_field_exists_in_metadata_parameters = {
    0: {
        "metadata": metadata_attributes_only,
        "field_name": "field_integer",
        "return_value": True,
    },
    1: {
        "metadata": metadata_attributes_only,
        "field_name": "foo",
        "return_value": False,
    },
    2: {
        "metadata": metadata_fr_dept_geometry_only,
        "field_name": "bar",
        "return_value": False,
    },
}

check_attributes_index_in_metadata_parameters = {
    0: {
        "metadata": metadata_fr_dept_population,
        "field_name": "CODE_DEPT",
        "type": "hashtable",
        "return_value": False,
    },
    1: {
        "metadata": add_attributes_index_in_metadata(
            **{
                "metadata": copy.deepcopy(metadata_fr_dept_population),
                "field_name": "CODE_DEPT",
                "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
            }
        ),
        "field_name": "CODE_DEPT",
        "type": "hashtable",
        "return_value": True,
    },
    2: {
        "metadata": add_attributes_index_in_metadata(
            **{
                "metadata": copy.deepcopy(metadata_fr_dept_population),
                "field_name": "CODE_DEPT",
                "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
            }
        ),
        "field_name": "CODE_DEPT",
        "type": "btree",
        "return_value": False,
    },
    3: {
        "metadata": add_attributes_index_in_metadata(
            **{
                "metadata": copy.deepcopy(metadata_fr_dept_population),
                "field_name": "CODE_DEPT",
                "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
            }
        ),
        "field_name": "FOO",
        "type": "hashtable",
        "return_value": field_missing.format(field_name="FOO"),
    },
    4: {
        "metadata": metadata_geometry_only_all_geometries_type,
        "field_name": "FOO",
        "type": "hashtable",
        "return_value": field_missing.format(field_name="FOO"),
    },
}

delete_attributes_index_in_metadata_parameters = {
    0: {
        "metadata": metadata_fr_dept_population,
        "field_name": "CODE_DEPT",
        "type": None,
        "return_value": field_name_not_indexing.format(field_name="CODE_DEPT"),
    },
    1: {
        "metadata": add_attributes_index_in_metadata(
            **{
                "metadata": copy.deepcopy(metadata_fr_dept_population),
                "field_name": "CODE_DEPT",
                "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
            }
        ),
        "field_name": "CODE_DEPT",
        "type": None,
        "return_value": metadata_fr_dept_population,
    },
    2: {
        "metadata": add_attributes_index_in_metadata(
            **{
                "metadata": copy.deepcopy(metadata_fr_dept_population),
                "field_name": "CODE_DEPT",
                "index": geolayer_fr_dept_population_CODE_DEPT_hash_index,
            }
        ),
        "field_name": "CODE_DEPT",
        "type": "btree",
        "return_value": field_name_not_indexing.format(field_name="CODE_DEPT"),
    },
}


def test_all():
    # drop_field_in_metadata
    print(test_function(drop_field_in_metadata, drop_field_in_metadata_parameters))

    # drop_field_that_not_exists_in_metadata
    print(
        test_function(
            drop_field_that_not_exists_in_metadata,
            drop_field_that_not_exists_in_metadata_parameters,
        )
    )

    # reorder_metadata_field_index_after_field_drop
    print(
        test_function(
            reorder_metadata_field_index_after_field_drop,
            reorder_metadata_field_index_after_field_drop_parameters,
        )
    )

    # rename_field_in_metadata_parameters
    print(test_function(rename_field_in_metadata, rename_field_in_metadata_parameters))

    # create_field_in_metadata
    print(test_function(create_field_in_metadata, create_field_in_metadata_parameters))

    # check_if_field_exists
    print(
        test_function(
            check_if_field_exists_in_metadata,
            check_if_field_exists_in_metadata_parameters,
        )
    )

    # add_attributes_index_in_metadata
    print(
        test_function(
            add_attributes_index_in_metadata,
            add_attributes_index_in_metadata_parameters,
        )
    )

    # check_attributes_index_in_metadata
    print(
        test_function(
            check_attributes_index_in_metadata,
            check_attributes_index_in_metadata_parameters,
        )
    )

    # delete_attributes_index_in_metadata
    print(
        test_function(
            delete_attributes_index_in_metadata,
            delete_attributes_index_in_metadata_parameters,
        )
    )


if __name__ == "__main__":
    test_all()



from geoformat.geoprocessing.measure.distance import (
    euclidean_distance,
    manhattan_distance,
    euclidean_distance_point_vs_segment,
    point_vs_segment_distance
)
from tests.utils.tests_utils import test_function

euclidean_distance_parameters = {
    0: {
        'point_a': (0, 0),
        'point_b': (0, 5),
        'return_value': 5
    },
    1:  {
        'point_a': (-5, -1),
        'point_b': (-5, -4),
        'return_value': 3
    },
    2:  {
        'point_a': (3, 2),
        'point_b': (9, 7),
        'return_value': 7.810249675906654
    },
    3:  {
        'point_a': (-2, -2),
        'point_b': (0, -4),
        'return_value': 2.8284271247461903
    },
    4:  {
        'point_a': [23.194, -20.28],
        'point_b': [2.38, 57.322],
        'return_value': 80.3448380420298
    },
}

manhattan_distance_parameters = {
    0: {
        'point_a': (0, 0),
        'point_b': (0, 5),
        'return_value': 5
    },
    1:  {
        'point_a': (-5, -1),
        'point_b': (-5, -4),
        'return_value': 3
    },
    2:  {
        'point_a': (3, 2),
        'point_b': (9, 7),
        'return_value': 11
    },
    3:  {
        'point_a': (-2, -2),
        'point_b': (0, -4),
        'return_value': 4
    },
}

euclidean_distance_point_vs_segment_parameters = {
    0: {
        'point': (0, 4),
        'segment': ((-5, 4), (5, 4)),
        'return_value': 0
    },
    1: {
        'point': (0, 3),
        'segment': ((-5, 4), (5, 4)),
        'return_value': 1
    },
    2: {
        'point': (0, 3),
        'segment': ((-5, 4), (5, 4)),
        'return_value': 1
    },
    3: {
        'point': (6, 3),
        'segment': ((-5, 4), (5, 4)),
        'return_value': 1.4142135623730951
    },
    4: {
        'point': [23.194, -20.28],
        'segment': [[2.38, 57.322], [2.38, 57.322]],
        'return_value': 80.3448380420298
    }
}

point_vs_segment_distance_parameters = {
    0: {
        'point': (0, 4),
        'segment': ((-5, 4), (5, 4)),
        'return_value': 0
    },
    1: {
        'point': (0, 3),
        'segment': ((-5, 4), (5, 4)),
        'return_value': 1
    },
    2: {
        'point': (6, 1),
        'segment': ((-5, 4), (5, 4)),
        'return_value': 3.1622776601683795
    },
    3: {
        'point': (6, 3),
        'segment': ((-5, 4), (5, 4)),
        'return_value': 1.4142135623730951
    },
    4: {
        'point': (0, 4),
        'segment': ((-5, 4), (5, 4)),
        'distance_function': manhattan_distance,
        'return_value': 0,
    },
    5: {
        'point': (0, 3),
        'segment': ((-5, 4), (5, 4)),
        'distance_function': manhattan_distance,
        'return_value': 1
    },
    6: {
        'point': (6, 1),
        'segment': ((-5, 4), (5, 4)),
        'distance_function': manhattan_distance,
        'return_value': 4
    },
    7: {
        'point': (6, 3),
        'segment': ((-5, 4), (5, 4)),
        'distance_function': manhattan_distance,
        'return_value': 2
    },
    8: {
        'point': [23.194, -20.28],
        'segment': [[2.38, 57.322], [2.38, 57.322]],
        'distance_function': euclidean_distance,
        'return_value': 80.3448380420298
    }
}


def test_all():
    # euclidean_distance
    print(test_function(euclidean_distance, euclidean_distance_parameters))

    # manhattan_distance
    print(test_function(manhattan_distance, manhattan_distance_parameters))

    # euclidean_distance_point_vs_segment
    print(test_function(euclidean_distance_point_vs_segment, euclidean_distance_point_vs_segment_parameters))

    # point_vs_segment_distance
    print(test_function(point_vs_segment_distance, point_vs_segment_distance_parameters))


if __name__ == '__main__':
    test_all()



import datetime

from geoformat.constraints.primary_key import create_pk

from tests.utils.tests_utils import test_function

from tests.data.geolayers import (
    geolayer_attributes_only_without_none_value,
    geolayer_fr_dept_geometry_only,
    geolayer_attributes_to_force_only_forced
)

from geoformat.conf.error_messages import (
    field_missing,
    non_unique_values
)

create_pk_parameters = {
    0: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_integer',
        "return_value": {586: 0}
    },
    1: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_integer_list',
        "return_value": {(5879, 8557): 0}
    },
    2: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_real',
        "return_value": {8789.97568: 0}
    },
    3: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_real_list',
        "return_value": {(89798.3654, 8757.97568): 0}
    },
    4: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_string',
        "return_value": {'salut': 0}
    },
    5: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_string_list',
        "return_value": {('bonjour', 'monsieur'): 0}
    },
    6: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_date',
        "return_value": {datetime.date(2020, 3, 31): 0}
    },
    7: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_time',
        "return_value": {datetime.time(11, 22, 10, 999): 0}
    },
    8: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_datetime',
        "return_value": {datetime.datetime(2020, 3, 31, 11, 22, 10, 999): 0}
    },
    9: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_binary',
        "return_value": {b'\x00\x00\x00\x00\x04\x00\x00\x00\x02\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01?\xf0\x00\x00\x00\x00\x00\x00?\xf0\x00\x00\x00\x00\x00\x00': 0}
    },
    10: {
        "geolayer": geolayer_attributes_only_without_none_value,
        "pk_field_name": 'field_boolean',
        "return_value": {True: 0}
    },
    11: {
        "geolayer": geolayer_fr_dept_geometry_only,
        "pk_field_name": 'foo',
        "return_value": field_missing.format(field_name='foo')
    },
    12: {
        "geolayer": geolayer_attributes_to_force_only_forced,
        "pk_field_name": "field_integer_list",
        "return_value": non_unique_values.format(field_name="field_integer_list")
    }
}


def test_all():

    # create_pk
    print(test_function(create_pk, create_pk_parameters))


if __name__ == '__main__':
    test_all()


from tests.utils.tests_utils import test_function

from tests.data.geometries import POINT, POINT_EMPTY, LINESTRING, LINESTRING_EMPTY, POLYGON, POLYGON_EMPTY, MULTIPOINT, MULTIPOINT_EMPTY, MULTILINESTRING, MULTILINESTRING_EMPTY, MULTIPOLYGON, MULTIPOLYGON_EMPTY, GEOMETRYCOLLECTION, GEOMETRYCOLLECTION_EMPTY

from geoformat.explore_data.duplicate import (
    get_feature_attributes_hash,
    get_feature_geometry_hash,
    get_feature_hash,
    get_duplicate_features
)

from geoformat.manipulation.geolayer_manipulation import feature_list_to_geolayer

from tests.data.features import (
    feature_dpt_data_only_a,
    feature_dpt_data_only_b,
    feature_dpt_data_only_c,
    feature_dpt_data_only_d,
    feature_dpt_geometry_only_a,
    feature_dpt_geometry_only_b,
    feature_dpt_geometry_only_c,
    feature_dpt_geometry_only_d,
    feature_dpt_data_and_geometry_a,
    feature_dpt_data_and_geometry_b,
    feature_dpt_data_and_geometry_c,
    feature_dpt_data_and_geometry_d,
)

get_feature_attributes_hash_parameters = {
    0: {
        "feature_attributes": {"CODE_DEPT": "53", "NOM_DEPT": "MAYENNE"},
        "field_attributes_field_name_order": ['CODE_DEPT', "NOM_DEPT"],
        "return_value": "51c7470e93695db9d24dea51653ee9a1249864a1a09c84fd9736e87080f75d0f",
    },
    1: {
        "feature_attributes": {"CODE_DEPT": "53", "NOM_DEPT": "MAYENNE"},
        "field_attributes_field_name_order": ['NOM_DEPT', "CODE_DEPT"],
        "return_value": "129cd2a582cd17950c10376e6242eebe6b817e163091b6cd17547017180cd9d6",
    },
    2: {
        "feature_attributes": {"NOM_DEPT": "MAYENNE", "CODE_DEPT": "53",},
        "field_attributes_field_name_order": ['CODE_DEPT', "NOM_DEPT"],
        "return_value": "51c7470e93695db9d24dea51653ee9a1249864a1a09c84fd9736e87080f75d0f",
    },
    3: {
        "feature_attributes": {"NOM_DEPT": "MAYENNE", "CODE_DEPT": "53", },
        "field_attributes_field_name_order": ['NOM_DEPT', "CODE_DEPT"],
        "return_value": "129cd2a582cd17950c10376e6242eebe6b817e163091b6cd17547017180cd9d6",
    },
    4: {
        "feature_attributes": {"NOM_DEPT": "MAYENNE", "CODE_DEPT": "53", },
        "field_attributes_field_name_order": [],
        "return_value": "4f53cda18c2baa0c0354bb5f9a3ecbe5ed12ab4d8e11ba873c2f11161202b945",
    },
    5: {
        "feature_attributes": {},
        "field_attributes_field_name_order": [],
        "return_value": "4f53cda18c2baa0c0354bb5f9a3ecbe5ed12ab4d8e11ba873c2f11161202b945",
    },
    6: {
        "feature_attributes": {},
        "field_attributes_field_name_order": ['NOM_DEPT', "CODE_DEPT"],
        "return_value": "b554fb2635a2b08395e00bee3abb78acdc3ca105b7844665ed1f6000b4ade489",
    },
    7: {
        "feature_attributes": None,
        "field_attributes_field_name_order": ['NOM_DEPT', "CODE_DEPT"],
        "return_value": "b554fb2635a2b08395e00bee3abb78acdc3ca105b7844665ed1f6000b4ade489",
    },
}

get_feature_geometry_hash_parameters = {
    0: {
        "feature_geometry": POINT,
        "return_value": "8fa3638269b284ff81154796e64c3622d766449fc711b21ae532a6d30dc674f7",
    },
    1: {
        "feature_geometry": POINT_EMPTY,
        "return_value": "95af905f4db3b819abc809b159456929f0e6b71cc7dba8eaa23992adcfe0f6c3",
    },
    2: {
        "feature_geometry": LINESTRING,
        "return_value": "b4cd3ad1b61a78511ef26d19aca752bba385b9a3abbe86d515e745ef0b104378",
    },
    3: {
        "feature_geometry": LINESTRING_EMPTY,
        "return_value": "3a40f051d7860323de7dc25da2f7c03f7727837c16e9c7a208c217e6c11ba57d",
    },
    4: {
        "feature_geometry": POLYGON,
        "return_value": "bfe28dbb4b9952b0a3ebb87f4e581197cab2f643e30e6912cbb8881247d33b8f",
    },
    5: {
        "feature_geometry": POLYGON_EMPTY,
        "return_value": "a10cdee8d37cc737efda4f4ecb273a62ef6281eb059831723a7f05db6ca91203",
    },
    6: {
        "feature_geometry": MULTIPOINT,
        "return_value": "5c79decd6b19e2f0ecd200f4459c0df3b40dd636258b571e42be398e48803c1d",
    },
    7: {
        "feature_geometry": MULTIPOINT_EMPTY,
        "return_value": "d04df4541745df552bf656601bd0ba5a803ff47be8c8d06e4f7e833f5fd72727",
    },

    8: {
        "feature_geometry": MULTILINESTRING,
        "return_value": "d59e25a3cf6c253494f13721088677bbe5077f5060d817a60cc71e9b837eeb97",
    },
    9: {
        "feature_geometry": MULTILINESTRING_EMPTY,
        "return_value": "b8db8e1a212398c530e340e0c8fb2b7bcfdc33639c64ee290a92acf9d52f4f27",
    },
    10: {
        "feature_geometry": MULTIPOLYGON,
        "return_value": "94594e71f626c43f8d871d64e10c88e86d32e4ec0b89fa3a5c0118f5ee1bc007",
    },
    11: {
        "feature_geometry": MULTIPOLYGON_EMPTY,
        "return_value": "9503e533b097397f5118a748890cd09caacfd4c95c4d13f1fd106a5b5e39e78a",
    },
    12: {
        "feature_geometry": GEOMETRYCOLLECTION,
        "return_value": "b6c9d62bdfcd0c440ff0a952b255957c6869b96fb532ad6e76f2dbac05af611d",
    },
    13: {
        "feature_geometry": GEOMETRYCOLLECTION_EMPTY,
        "return_value": "fde666df544418c9cc1c2d036f1e0677658e66d56a2c40241290404e61567404",
    },
    14: {
        "feature_geometry": {},
        "return_value": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
    },
    15: {
        "feature_geometry": {},
        "return_value": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
    },

}

get_feature_hash_parameters = {
    0: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}},
        "field_attributes_field_name_order": ["id", "name"],
        "attribute_hash": True,
        "geometry_hash": True,
        "return_value": "1abfc0b0962e2f8f0bc269edc68b7bdee90b0ccae602d811ee2e5c84d82e6bcc"
    },
    1: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}},
        "field_attributes_field_name_order": None,
        "attribute_hash": True,
        "geometry_hash": True,
        "return_value": "variable field_attributes_field_name_order must be fill"
    },
    2: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}},
        "field_attributes_field_name_order": ["name", "id"],
        "attribute_hash": True,
        "geometry_hash": True,
        "return_value": "1abee36b192e37746120c85c1abc88a22af548aa37bf82f6639b31e29f8b6614"
    },
    3: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}},
        "field_attributes_field_name_order": ["id", "name"],
        "attribute_hash": True,
        "geometry_hash": False,
        "return_value": "1abfc0b0962e2f8f0bc269edc68b7bdee90b0ccae602d811ee2e5c84d82e6bcc"
    },
    4: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}},
        "field_attributes_field_name_order": ["id", "name"],
        "attribute_hash": False,
        "geometry_hash": True,
        "return_value": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
    },
    5: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}},
        "field_attributes_field_name_order": ["id", "name"],
        "attribute_hash": False,
        "geometry_hash": False,
        "return_value": "At least one of this variables (attribute_hash or geometry_hash) must be True"
    },
    6: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}, "geometry": POINT},
        "field_attributes_field_name_order": ["id", "name"],
        "attribute_hash": True,
        "geometry_hash": True,
        "return_value": "4fd3977f22544bc2127918097464827828a7c856052fddba6647c61b7fa0328c"
    },
    7: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}, "geometry": POINT},
        "field_attributes_field_name_order": ["id", "name"],
        "attribute_hash": True,
        "geometry_hash": False,
        "return_value": "1abfc0b0962e2f8f0bc269edc68b7bdee90b0ccae602d811ee2e5c84d82e6bcc"
    },
    8: {
        "feature": {"attributes": {"id": 0, "name": "Alice"}, "geometry": POINT},
        "field_attributes_field_name_order": ["id", "name"],
        "attribute_hash": False,
        "geometry_hash": True,
        "return_value": "f1d248b203635aece53e123298ee6779a2d06df0ab68078c02bd6ef6f72af70c"
    },
    9: {
        "feature": {"geometry": POINT},
        "field_attributes_field_name_order": None,
        "attribute_hash": True,
        "geometry_hash": True,
        "return_value": "f1d248b203635aece53e123298ee6779a2d06df0ab68078c02bd6ef6f72af70c"
    },
    10: {
        "feature": {"geometry": POINT},
        "field_attributes_field_name_order": None,
        "attribute_hash": True,
        "geometry_hash": False,
        "return_value": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
    },
}

get_duplicate_features_parameters = {
    0: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_only_a,
            feature_dpt_data_only_b,
            feature_dpt_data_only_c,
            feature_dpt_data_only_d,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": ()
    },
    1: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_only_a,
            feature_dpt_data_only_c,
            feature_dpt_data_only_c,
            feature_dpt_data_only_d,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": (2,)
    },
    2: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_only_a,
            feature_dpt_data_only_c,
            feature_dpt_data_only_c,
            feature_dpt_data_only_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": (2, 3)
    },
    3: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_only_a,
            feature_dpt_data_only_c,
            feature_dpt_data_only_c,
            feature_dpt_data_only_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": False,
        "check_geometry_duplicate": True,
        "return_value": (1, 2, 3)
    },
    4: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_only_a,
            feature_dpt_data_only_c,
            feature_dpt_data_only_c,
            feature_dpt_data_only_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": False,
        "return_value": (2, 3)
    },
    5: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_geometry_only_a,
            feature_dpt_geometry_only_b,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_d,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": ()
    },
    6: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_geometry_only_a,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_d,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": (2,)
    },
    7: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_geometry_only_a,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": (2, 3)
    },
    8: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_geometry_only_a,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": False,
        "check_geometry_duplicate": True,
        "return_value": (2, 3)
    },
    9: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_geometry_only_a,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_c,
            feature_dpt_geometry_only_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": False,
        "return_value": (1, 2, 3)
    },
    10: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_and_geometry_a,
            feature_dpt_data_and_geometry_b,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_d,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": ()
    },
    11: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_and_geometry_a,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_d,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": (2,)
    },
    12: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_and_geometry_a,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": (2, 3)
    },
    13: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_and_geometry_a,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": False,
        "check_geometry_duplicate": True,
        "return_value": (2, 3)
    },
    14: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            feature_dpt_data_and_geometry_a,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_c,
            feature_dpt_data_and_geometry_c,
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": False,
        "return_value": (2, 3)
    },
    15: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            {"attributes": {"id": 0, "name": "Alice"}, "geometry": POINT},
            {"attributes": {"id": None, "name": "Bob"}, "geometry": LINESTRING},
            {"attributes": {"id": None, "name": "Patrick"}, "geometry": LINESTRING},
            {"attributes": {"id": 1, "name": "Jane"}, "geometry": POINT},
            {"attributes": {"id": 1, "name": "Jane"}, "geometry": POLYGON},
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": True,
        "return_value": ()
    },
    16: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            {"attributes": {"id": 0, "name": "Alice"}, "geometry": POINT},
            {"attributes": {"id": None, "name": "Bob"}, "geometry": LINESTRING},
            {"attributes": {"id": None, "name": "Patrick"}, "geometry": LINESTRING},
            {"attributes": {"id": 1, "name": "Jane"}, "geometry": POINT},
            {"attributes": {"id": 1, "name": "Jane"}, "geometry": POLYGON},
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": False,
        "check_geometry_duplicate": True,
        "return_value": (2, 3)
    },
    17: {
        "geolayer": feature_list_to_geolayer(feature_list=[
            {"attributes": {"id": 0, "name": "Alice"}, "geometry": POINT},
            {"attributes": {"id": None, "name": "Bob"}, "geometry": LINESTRING},
            {"attributes": {"id": None, "name": "Patrick"}, "geometry": LINESTRING},
            {"attributes": {"id": 1, "name": "Jane"}, "geometry": POINT},
            {"attributes": {"id": 1, "name": "Jane"}, "geometry": POLYGON},
            ],
            geolayer_name='test'
        ),
        "check_attribute_duplicate": True,
        "check_geometry_duplicate": False,
        "return_value": (4,)
    },
}
def test_all():

    # get_feature_attributes_hash
    print(test_function(get_feature_attributes_hash, get_feature_attributes_hash_parameters))

    # get_feature_geometry_hash
    print(test_function(get_feature_geometry_hash, get_feature_geometry_hash_parameters))

    # get_feature_hash
    print(test_function(get_feature_hash, get_feature_hash_parameters))

    # get_duplicate_features
    print(test_function(get_duplicate_features, get_duplicate_features_parameters))

if __name__ == '__main__':
    test_all()



from tests.data.geolayers import geolayer_attributes_only
from geoformat.driver.ogr.ogr_driver import geolayer_to_ogr_layer, ogr_layer_to_geolayer
from tests.utils.compare import compare_geolayer


def test_postgresql_attributes(attributes_geolayer):
    host = 'localhost'
    database = 'test'
    user = 'postgres'
    password = 'postgres'
    port = 5432

    pg = "PG: host={host} dbname={db_name} user={user} password={password} port={port}"
    pg_connect = pg.format(host=host, db_name=database, user=user, password=password, port=port)

    path_geolayer_attirbutes_only = pg_connect

    # write file
    geolayer_to_ogr_layer(
        geolayer=attributes_geolayer,
        path=path_geolayer_attirbutes_only,
        driver_name='postgresql',
        ogr_options=['OVERWRITE=YES']
    )

    # open created just bellow geojson to geolayer (to compare to geolayer_attributes_only)
    ogr_geolayer_attributes = ogr_layer_to_geolayer(
        path=path_geolayer_attirbutes_only,
        layer_id_or_name=attributes_geolayer['metadata']['name']
    )

    # compare both geolayer
    compare_geolayer(attributes_geolayer, ogr_geolayer_attributes)


if __name__ == '__main__':
    test_postgresql_attributes(attributes_geolayer=geolayer_attributes_only)




from geoformat.geoprocessing.connectors.operations import (
    coordinates_to_point,
    coordinates_to_segment,
    coordinates_to_bbox,
    segment_to_bbox
)


from tests.data.geometries import (
    POINT,
    MULTIPOINT,
    LINESTRING,
    MULTILINESTRING,
    POLYGON,
    MULTIPOLYGON
)

from tests.data.coordinates import (
    linestring_coordinates,
    multilinestring_coordinates,
    polygon_coordinates,
    multipolygon_coordinates
)

from tests.data.segments import (
    segment_a,
    segment_c,
    segment_d,
    segment_b,
    segment_e,
    segment_f,
    segment_g,
    segment_h,
    segment_i,


)

from tests.utils.tests_utils import test_function

coordinates_to_point_parameters = {
    0: {
        'coordinates': POINT['coordinates'],
        'return_value': ([-115.81, 37.24], ),
    },
    1: {
        'coordinates': MULTIPOINT['coordinates'],
        'return_value': ([-155.52, 19.61], [-156.22, 20.74], [-157.97, 21.46]),
    },
    2: {
        'coordinates': LINESTRING['coordinates'],
        'return_value': ([8.919, 44.4074], [8.923, 44.4075]),
    },
    3: {
        'coordinates': MULTILINESTRING['coordinates'],
        'return_value': ([3.75, 9.25], [-130.95, 1.52], [23.15, -34.25], [-1.35, -4.65], [3.45, 77.95]),
    },
    4: {
        'coordinates': POLYGON['coordinates'],
        'return_value': ([2.38, 57.322], [23.194, -20.28], [-120.43, 19.15], [2.38, 57.322], [-5.21, 23.51], [15.21, -10.81], [-20.51, 1.51], [-5.21, 23.51])
    },
    5: {
        'coordinates': MULTIPOLYGON['coordinates'],
        'return_value': ([3.78, 9.28], [-130.91, 1.52], [35.12, 72.234], [3.78, 9.28], [23.18, -34.29], [-1.31, -4.61], [3.41, 77.91], [23.18, -34.29]),
    },
}

coordinates_to_segment_parameters = {
    0: {
        'coordinates': linestring_coordinates,
        'return_value': ([[8.919, 44.4074], [8.923, 44.4075]],)
    },
    1: {
        'coordinates': multilinestring_coordinates,
        'return_value': ([[3.75, 9.25], [-130.95, 1.52]], [[23.15, -34.25], [-1.35, -4.65]], [[-1.35, -4.65], [3.45, 77.95]])
    },
    2: {
        'coordinates': polygon_coordinates,
        'return_value': ([[2.38, 57.322], [23.194, -20.28]], [[23.194, -20.28], [-120.43, 19.15]], [[-120.43, 19.15], [2.38, 57.322]], [[-5.21, 23.51], [15.21, -10.81]], [[15.21, -10.81], [-20.51, 1.51]], [[-20.51, 1.51], [-5.21, 23.51]])
    },
    3: {
        'coordinates': multipolygon_coordinates,
        'return_value': ([[3.78, 9.28], [-130.91, 1.52]], [[-130.91, 1.52], [35.12, 72.234]], [[35.12, 72.234], [3.78, 9.28]], [[23.18, -34.29], [-1.31, -4.61]], [[-1.31, -4.61], [3.41, 77.91]], [[3.41, 77.91], [23.18, -34.29]])
    },
}

coordinates_to_bbox_parameters = {
    0: {
        'coordinates': POINT['coordinates'],
        'return_value': (-115.81, 37.24, -115.81, 37.24),
    },
    1: {
        'coordinates': MULTIPOINT['coordinates'],
        'return_value': (-157.97, 19.61, -155.52, 21.46)
    },
    2: {
        'coordinates': LINESTRING['coordinates'],
        'return_value': (8.919, 44.4074, 8.923, 44.4075),
    },
    3: {
        'coordinates': MULTILINESTRING['coordinates'],
        'return_value': (-130.95, -34.25, 23.15, 77.95),
    },
    4: {
        'coordinates': POLYGON['coordinates'],
        'return_value': (-120.43, -20.28, 23.194, 57.322)
    },
    5: {
        'coordinates': MULTIPOLYGON['coordinates'],
        'return_value': (-130.91, -34.29, 35.12, 77.91)
    },
}

segment_to_bbox_parameters = {
    0: {
        'segment': segment_a,
        'return_value': (0, 0, 0, 1),
    },
    1: {
        'segment': segment_b,
        'return_value': (0, 0, 0, 1),
    },
    2: {
        'segment': segment_c,
        'return_value': (0, -1, 0, 0),
    },
    3: {
        'segment': segment_d,
        'return_value': (0, -1, 0, 0),
    },
    4: {
        'segment': segment_e,
        'return_value': (-1, -1, 1, 1),
    },
    5: {
        'segment': segment_f,
        'return_value': (-1, -1, 1, 1),
    },
    6: {
        'segment': segment_g,
        'return_value': (-1, -1, 1, 1),
    },
    7: {
        'segment': segment_h,
        'return_value': (-1, -1, -1, 1),
    },
    8: {
        'segment': segment_i,
        'return_value': (1, -1, 1, 1),
    },
}


def test_all():
    # coordinates_to_point
    print(test_function(coordinates_to_point, coordinates_to_point_parameters))

    # coordinates_to_segment
    print(test_function(coordinates_to_segment, coordinates_to_segment_parameters))

    # coordinates_to_bbox
    print(test_function(coordinates_to_bbox, coordinates_to_bbox_parameters))

    # segment_to_bbox
    print(test_function(segment_to_bbox, segment_to_bbox_parameters))


if __name__ == '__main__':
    test_all()



from pathlib import Path

from tests.utils.tests_utils import test_function

from geoformat.conf.path import (
    add_extension_path,
    path_to_file_path,
    path_is_file,
    verify_input_path_is_file,
    path_is_http,
    verify_input_path_is_http,
    path_is_dir,
    path_not_a_dir,
    verify_input_path_is_dir,
    open_http_path,
)

from geoformat.conf.error_messages import (
    path_not_valid,
    path_http_not_valid,
    path_not_http,
)
from http.client import HTTPResponse

add_extension_path_parameters = {
    0: {
        "path": Path("data/geojson/test"),
        "add_extension": None,
        "return_value": Path("data/geojson/test"),
    },
    1: {
        "path": Path("data/geojson/test"),
        "add_extension": ".geojson",
        "return_value": Path("data/geojson/test.geojson"),
    },
    2: {
        "path": Path("data/geojson/test.geojson"),
        "add_extension": ".kml",
        "return_value": Path("data/geojson/test.geojson.kml"),
    },
}

path_is_file_parameters = {
    0: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath(
            "geoformat/conf/path.py"
        ),
        "return_value": True,
    },
    1: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath("geoformat/conf"),
        "return_value": False,
    },
    2: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath("geoformat/foo"),
        "return_value": False,
    },
}

verify_input_path_is_file_parameters = {
    0: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath(
            "geoformat/conf/path.py"
        ),
        "return_value": Path(__file__).parent.parent.parent.parent.joinpath(
            "geoformat/conf/path.py"
        ),
    },
    1: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath("geoformat/conf"),
        "return_value": path_not_valid.format(
            path=Path(__file__).parent.parent.parent.parent.joinpath("geoformat/conf")
        ),
    },
    2: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath("geoformat/foo"),
        "return_value": path_not_valid.format(
            path=Path(__file__).parent.parent.parent.parent.joinpath("geoformat/foo")
        ),
    },
}

open_http_path_parameters = {
    0: {
        "path": "https://foo/bar",
        "headers": None,
        "return_value": "<urlopen error [Errno -3] Temporary failure in name resolution>",
    },
    1: {
        "path": "https://france-geojson.gregoiredavid.fr/repo/regions.geojson",
        "headers": None,
        "return_value": HTTPResponse,
    },
}

path_is_http_parameters = {
    0: {
        "path": "https://france-geojson.gregoiredavid.fr/repo/regions.geojson",
        "headers": None,
        "return_value": (True, 200),
    },
    1: {
        "path": "https://raw.githubusercontent.com/mbloch/mapshaper/master/test/data/three_points.geojson",
        "headers": None,
        "return_value": (True, 200),
    },
    2: {
        "path": "https://france-geojson.gregoiredavid.fr/repo/toto.geojson",
        "headers": None,
        "return_value": (False, 404),
    },
    3: {
        "path": "https://foo/bar",
        "headers": None,
        "return_value": (False, 404),
    },
    4: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath(
            "geoformat/conf/path.py"
        ),
        "headers": None,
        "return_value": (False, None),
    },
}

path_is_dir_parameters = {
    0: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath(
            "geoformat/conf/path.py"
        ),
        "return_value": False,
    },
    1: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath("geoformat/conf"),
        "return_value": True,
    },
    2: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath("geoformat/foo"),
        "return_value": False,
    },
}

verify_input_path_is_dir_parameters = {
    0: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath(
            "geoformat/conf/path.py"
        ),
        "return_value": path_not_a_dir.format(
            path=Path(__file__).parent.parent.parent.parent.joinpath(
                "geoformat/conf/path.py"
            )
        ),
    },
    1: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath("geoformat/conf"),
        "return_value": Path(__file__).parent.parent.parent.parent.joinpath(
            "geoformat/conf"
        ),
    },
    2: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath("geoformat/foo"),
        "return_value": path_not_a_dir.format(
            path=Path(__file__).parent.parent.parent.parent.joinpath("geoformat/foo")
        ),
    },
}

verify_input_path_is_http_parameters = {
    0: {
        "path": "https://france-geojson.gregoiredavid.fr/repo/regions.geojson",
        "headers": None,
        "return_value": "https://france-geojson.gregoiredavid.fr/repo/regions.geojson",
    },
    1: {
        "path": "https://raw.githubusercontent.com/mbloch/mapshaper/master/test/data/three_points.geojson",
        "headers": None,
        "return_value": "https://raw.githubusercontent.com/mbloch/mapshaper/master/test/data/three_points.geojson",
    },
    2: {
        "path": "https://france-geojson.gregoiredavid.fr/repo/toto.geojson",
        "headers": None,
        "return_value": path_http_not_valid.format(
            path="https://france-geojson.gregoiredavid.fr/repo/toto.geojson", code=404
        ),
    },
    3: {
        "path": "https://foo/bar",
        "headers": None,
        "return_value": path_http_not_valid.format(path="https://foo/bar", code=404),
    },
    4: {
        "path": Path(__file__).parent.parent.parent.parent.joinpath(
            "geoformat/conf/path.py"
        ),
        "headers": None,
        "return_value": path_not_http.format(
            path=Path(__file__).parent.parent.parent.parent.joinpath(
                "geoformat/conf/path.py"
            )
        ),
    },
}

path_to_file_path_parameters = {
    0: {
        "path": Path(__file__).parent.parent.parent.joinpath("data").as_posix(),
        "geolayer_name": "test",
        "overwrite": True,
        "add_extension": None,
        "return_value": Path(__file__).parent.parent.parent.joinpath("data/test"),
    },
    1: {
        "path": Path(__file__).parent.parent.parent.joinpath("data/test").as_posix(),
        "geolayer_name": "test",
        "overwrite": True,
        "add_extension": None,
        "return_value": Path(__file__).parent.parent.parent.joinpath("data/test"),
    },
    2: {
        "path": Path(__file__).parent.parent.parent.joinpath("data/test").as_posix(),
        "geolayer_name": "test",
        "overwrite": True,
        "add_extension": ".geojson",
        "return_value": Path(__file__).parent.parent.parent.joinpath(
            "data/test.geojson"
        ),
    },
    3: {
        "path": Path(__file__).parent.parent.parent.joinpath("data"),
        "geolayer_name": "test",
        "overwrite": True,
        "add_extension": None,
        "return_value": Path(__file__).parent.parent.parent.joinpath("data/test"),
    },
    4: {
        "path": Path(__file__).parent.parent.parent.joinpath("data/test"),
        "geolayer_name": "test",
        "overwrite": True,
        "add_extension": None,
        "return_value": Path(__file__).parent.parent.parent.joinpath("data/test"),
    },
    5: {
        "path": Path(__file__).parent.parent.parent.joinpath("data/test"),
        "geolayer_name": "test",
        "overwrite": True,
        "add_extension": ".geojson",
        "return_value": Path(__file__).parent.parent.parent.joinpath(
            "data/test.geojson"
        ),
    },
}


def test_all():

    # add_extension_path
    print(test_function(add_extension_path, add_extension_path_parameters))

    # path_is_file
    print(test_function(path_is_file, path_is_file_parameters))

    # verify_input_path_is_file
    print(
        test_function(verify_input_path_is_file, verify_input_path_is_file_parameters)
    )

    # open_http_path
    print(test_function(open_http_path, open_http_path_parameters))

    # path_is_http
    print(test_function(path_is_http, path_is_http_parameters))

    # path_is_dir
    print(test_function(path_is_dir, path_is_dir_parameters))

    # verify_input_path_is_dir
    print(test_function(verify_input_path_is_dir, verify_input_path_is_dir_parameters))

    # verify_input_path_is_http
    print(
        test_function(verify_input_path_is_http, verify_input_path_is_http_parameters)
    )

    # path_to_file_path
    print(test_function(path_to_file_path, path_to_file_path_parameters))


if __name__ == "__main__":
    test_all()



import copy
from tests.utils.tests_utils import test_function

from geoformat.conf.error_messages import (
    variable_wrong_formatting,
    field_exists,
    metadata_geometry_ref_not_found,
    metadata_geometry_ref_type_not_match,
    geometry_ref_feature,
)


from geoformat.processing.data.join.merge_objects import (
    merge_metadata,
    merge_feature,
)

from tests.data.metadata import (
    metadata_fr_dept_data_and_geometry,
    metadata_fr_dept_data_only,
    metadata_fr_dept_population,
    metadata_fr_dept_population_geometry,
)

from tests.data.features import (
    feature_dpt_data_only_a,
    feature_dpt_geometry_only_a,
    feature_dpt_data_and_geometry_a,
    feature_dpt_population_a,
    feature_dpt_population_geometry_a,
)

merge_metadata_parameters = {
    0: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_and_geometry),
        "metadata_b": copy.deepcopy(metadata_fr_dept_data_and_geometry),
        "geolayer_name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "geometry_ref": "metadata_a",
        "rename_output_field_from_geolayer_a": "auto",
        "rename_output_field_from_geolayer_b": "auto",
        "return_value": {
            "metadata": {
                "name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
                "fields": {
                    "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                    "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                    "CODE_DEPT1": {"type": "String", "width": 2, "index": 2},
                    "NOM_DEPT1": {"type": "String", "width": 23, "index": 3},
                },
                "geometry_ref": {"type": {"Polygon", "MultiPolygon"}, "crs": 2154},
            },
            "fields_correspondance_a": {},
            "fields_correspondance_b": {"CODE_DEPT": "CODE_DEPT1", "NOM_DEPT": "NOM_DEPT1"}
        },
    },
    1: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_only),
        "metadata_b": copy.deepcopy(metadata_fr_dept_data_and_geometry),
        "geolayer_name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "geometry_ref": "metadata_a",
        "rename_output_field_from_geolayer_a": "auto",
        "rename_output_field_from_geolayer_b": "auto",
        "return_value": {
            "metadata": {
                "name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
                "fields": {
                    "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                    "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                    "CODE_DEPT1": {"type": "String", "width": 2, "index": 2},
                    "NOM_DEPT1": {"type": "String", "width": 23, "index": 3},
                },
            },
            "fields_correspondance_a": {},
            "fields_correspondance_b": {"CODE_DEPT": "CODE_DEPT1", "NOM_DEPT": "NOM_DEPT1"}
        },
    },
    2: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_only),
        "metadata_b": copy.deepcopy(metadata_fr_dept_data_and_geometry),
        "geolayer_name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "geometry_ref": "metadata_a",
        "rename_output_field_from_geolayer_a": "auto",
        "rename_output_field_from_geolayer_b": "auto",
        "return_value": {
            "metadata": {
                "name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
                "fields": {
                    "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                    "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                    "CODE_DEPT1": {"type": "String", "width": 2, "index": 2},
                    "NOM_DEPT1": {"type": "String", "width": 23, "index": 3},
                },
            },
            "fields_correspondance_a": {},
            "fields_correspondance_b": {"CODE_DEPT": "CODE_DEPT1", "NOM_DEPT": "NOM_DEPT1"}
        },
    },
    3: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_only),
        "metadata_b": copy.deepcopy(metadata_fr_dept_data_and_geometry),
        "geolayer_name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "geometry_ref": "metadata_b",
        "rename_output_field_from_geolayer_a": "auto",
        "rename_output_field_from_geolayer_b": "auto",
        "return_value": {
            "metadata": {
                "name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
                "fields": {
                    "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                    "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                    "CODE_DEPT1": {"type": "String", "width": 2, "index": 2},
                    "NOM_DEPT1": {"type": "String", "width": 23, "index": 3},
                },
                "geometry_ref": {"type": {"MultiPolygon", "Polygon"}, "crs": 2154},
            },
            "fields_correspondance_a": {},
            "fields_correspondance_b": {"CODE_DEPT": "CODE_DEPT1", "NOM_DEPT": "NOM_DEPT1"}
        },
    },
    4: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_only),
        "metadata_b": copy.deepcopy(metadata_fr_dept_data_and_geometry),
        "geolayer_name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
        "field_name_filter_a": [],
        "field_name_filter_b": ["CODE_DEPT", "NOM_DEPT"],
        "geometry_ref": "metadata_b",
        "rename_output_field_from_geolayer_a": "auto",
        "rename_output_field_from_geolayer_b": "auto",
        "return_value": {
            "metadata": {
                "name": "merge_FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
                "fields": {
                    "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                    "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                },
                "geometry_ref": {"type": {"Polygon", "MultiPolygon"}, "crs": 2154},
            },
            "fields_correspondance_a": {},
            "fields_correspondance_b": {},
        },
    },
    5: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_only),
        "metadata_b": copy.deepcopy(metadata_fr_dept_data_only),
        "geolayer_name": "FRANCE_DPT",
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "geometry_ref": "metadata_b",
        "rename_output_field_from_geolayer_a": {"CODE_DEPT": "id_dept", "NOM_DEPT": "name"},
        "rename_output_field_from_geolayer_b": {"CODE_DEPT": "CODE_DPT", "NOM_DEPT": "nom"},
        "return_value": {
            "metadata": {
                "name": "FRANCE_DPT",
                "fields": {
                    "id_dept": {"type": "String", "width": 2, "index": 0},
                    "name": {"type": "String", "width": 23, "index": 1},
                    "CODE_DPT": {"type": "String", "width": 2, "index": 2},
                    "nom": {"type": "String", "width": 23, "index": 3},
                },
            },
            "fields_correspondance_a": {"CODE_DEPT": "id_dept", "NOM_DEPT": "name"},
            "fields_correspondance_b": {"CODE_DEPT": "CODE_DPT", "NOM_DEPT": "nom"},
        },
    },
    6: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_only),
        "metadata_b": copy.deepcopy(metadata_fr_dept_data_only),
        "geolayer_name": "FRANCE_DPT",
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "geometry_ref": "metadata_b",
        "rename_output_field_from_geolayer_a": {"NOM_DEPT": "name"},
        "rename_output_field_from_geolayer_b": {"NOM_DEPT": "nom"},
        "return_value": field_exists.format(field_name="CODE_DEPT"),
    },
    7: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_only),
        "metadata_b": copy.deepcopy(metadata_fr_dept_population),
        "geolayer_name": "FRANCE_DPT",
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "geometry_ref": "metadata_b",
        "rename_output_field_from_geolayer_a": "auto",
        "rename_output_field_from_geolayer_b": "auto",
        "return_value": {
            "metadata": {
                "name": "FRANCE_DPT",
                "fields": {
                    "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                    "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                    "CODE_DEPT1": {"type": "String", "width": 2, "index": 2},
                    "INSEE_REG": {"type": "String", "width": 2, "index": 3},
                    "POPULATION": {"type": "Integer", "index": 4},
                    "AREA": {"type": "Real", "width": 7, "precision": 2, "index": 5},
                    "DENSITY": {"type": "Real", "width": 7, "precision": 2, "index": 6},
                },
            },
            "fields_correspondance_a": {},
            "fields_correspondance_b": {"CODE_DEPT": "CODE_DEPT1"},
        },
    },
    8: {
        "metadata_a": copy.deepcopy(metadata_fr_dept_data_and_geometry),
        "metadata_b": copy.deepcopy(metadata_fr_dept_population),
        "geolayer_name": "FRANCE_DPT_WITH_POPULATION",
        "field_name_filter_a": ["CODE_DEPT", "NOM_DEPT"],
        "field_name_filter_b": ["POPULATION", "DENSITY"],
        "geometry_ref": "metadata_a",
        "rename_output_field_from_geolayer_a": "auto",
        "rename_output_field_from_geolayer_b": "auto",
        "return_value": {
            "metadata": metadata_fr_dept_population_geometry,
            "fields_correspondance_a": {},
            "fields_correspondance_b": {},
        },
    },
}

merge_feature_parameters = {
    0: {
        "feature_a": copy.deepcopy(
            {
                "geometry": {
                    "type": feature_dpt_geometry_only_a["geometry"]["type"],
                    "coordinates": feature_dpt_geometry_only_a["geometry"][
                        "coordinates"
                    ],
                }
            }
        ),
        "feature_b": copy.deepcopy(feature_dpt_data_only_a),
        "merge_metadata": None,
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_a",
        "return_value": feature_dpt_data_and_geometry_a,
    },
    1: {
        "feature_a": copy.deepcopy(feature_dpt_data_only_a),
        "feature_b": copy.deepcopy(feature_dpt_geometry_only_a),
        "merge_metadata": None,
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_a",
        "return_value": feature_dpt_data_only_a,
    },
    2: {
        "feature_a": copy.deepcopy(feature_dpt_data_only_a),
        "feature_b": copy.deepcopy(
            {
                "geometry": {
                    "type": feature_dpt_geometry_only_a["geometry"]["type"],
                    "coordinates": feature_dpt_geometry_only_a["geometry"][
                        "coordinates"
                    ],
                }
            }
        ),
        "merge_metadata": None,
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_b",
        "return_value": feature_dpt_data_and_geometry_a,
    },
    3: {
        "feature_a": copy.deepcopy(feature_dpt_data_only_a),
        "feature_b": copy.deepcopy(feature_dpt_population_a),
        "merge_metadata": None,
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_a",
        "return_value": field_exists.format(field_name="CODE_DEPT"),
    },
    4: {
        "feature_a": copy.deepcopy(feature_dpt_data_only_a),
        "feature_b": copy.deepcopy(feature_dpt_population_a),
        "merge_metadata": None,
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "rename_fields_a": None,
        "rename_fields_b": {"CODE_DEPT": "CODE_DEPT1"},
        "geometry_ref": "feature_a",
        "return_value": {
            "attributes": {
                "CODE_DEPT": "53",
                "NOM_DEPT": "MAYENNE",
                "INSEE_REG": "52",
                "POPULATION": 307445,
                "AREA": 5208.37,
                "DENSITY": 59.03,
                "CODE_DEPT1": "53",
            }
        },
    },
    5: {
        "feature_a": copy.deepcopy(feature_dpt_data_only_a),
        "feature_b": copy.deepcopy(feature_dpt_population_a),
        "merge_metadata": {
            "name": "FRANCE_DPT_WITH_POPULATION",
            "fields": {
                "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                "POPULATION": {"type": "Integer", "index": 2},
                "DENSITY": {"type": "Real", "width": 7, "precision": 2, "index": 3},
            },
        },
        "field_name_filter_a": ["CODE_DEPT", "NOM_DEPT"],
        "field_name_filter_b": ["POPULATION", "DENSITY"],
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_a",
        "return_value": {
            "attributes": {
                "CODE_DEPT": "53",
                "NOM_DEPT": "MAYENNE",
                "POPULATION": 307445,
                "DENSITY": 59.03,
            }
        },
    },
    6: {
        "feature_a": copy.deepcopy(feature_dpt_population_a),
        "feature_b": copy.deepcopy(feature_dpt_geometry_only_a),
        "merge_metadata": {
            "name": "FRANCE_DPT_WITH_POPULATION",
            "fields": {
                "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                "POPULATION": {"type": "Integer", "index": 2},
                "DENSITY": {"type": "Real", "width": 7, "precision": 2, "index": 3},
            },
        },
        "field_name_filter_a": ["CODE_DEPT", "NOM_DEPT", "POPULATION", "DENSITY"],
        "field_name_filter_b": [],
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_b",
        "return_value": metadata_geometry_ref_not_found,
    },
    7: {
        "feature_a": copy.deepcopy(feature_dpt_population_a),
        "feature_b": copy.deepcopy(
            {
                "geometry": {
                    "type": feature_dpt_geometry_only_a["geometry"]["type"],
                    "coordinates": feature_dpt_geometry_only_a["geometry"][
                        "coordinates"
                    ],
                }
            }
        ),
        "merge_metadata": {
            "name": "FRANCE_DPT_WITH_POPULATION",
            "fields": {
                "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                "POPULATION": {"type": "Integer", "index": 2},
                "DENSITY": {"type": "Real", "width": 7, "precision": 2, "index": 3},
            },
            "geometry_ref": {"type": {"Polygon"}},
        },
        "field_name_filter_a": ["CODE_DEPT", "POPULATION", "DENSITY"],
        "field_name_filter_b": None,
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_b",
        "return_value": {
            "attributes": {"CODE_DEPT": "53", "POPULATION": 307445, "DENSITY": 59.03},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [399495.0, 6830885.0],
                        [398130.0, 6822559.0],
                        [400321.0, 6810723.0],
                        [395852.0, 6803336.0],
                        [398626.0, 6784333.0],
                        [400465.0, 6781914.0],
                        [400197.0, 6773697.0],
                        [394099.0, 6773357.0],
                        [390140.0, 6770978.0],
                        [386941.0, 6760260.0],
                        [382932.0, 6754022.0],
                        [389872.0, 6749698.0],
                        [393110.0, 6750366.0],
                        [402067.0, 6747685.0],
                        [404251.0, 6751414.0],
                        [412442.0, 6746090.0],
                        [419671.0, 6744167.0],
                        [429458.0, 6743442.0],
                        [440863.0, 6746201.0],
                        [446732.0, 6745443.0],
                        [446459.0, 6750432.0],
                        [442128.0, 6753611.0],
                        [448124.0, 6758669.0],
                        [447308.0, 6764356.0],
                        [455060.0, 6767070.0],
                        [451057.0, 6776681.0],
                        [459373.0, 6778102.0],
                        [460615.0, 6783387.0],
                        [458409.0, 6789055.0],
                        [466280.0, 6794064.0],
                        [465298.0, 6799724.0],
                        [467628.0, 6811401.0],
                        [473893.0, 6813452.0],
                        [474394.0, 6821359.0],
                        [467262.0, 6822174.0],
                        [466087.0, 6830999.0],
                        [463434.0, 6833996.0],
                        [457920.0, 6827997.0],
                        [451256.0, 6826715.0],
                        [446687.0, 6829012.0],
                        [441174.0, 6828584.0],
                        [437568.0, 6825109.0],
                        [429868.0, 6822252.0],
                        [422197.0, 6821752.0],
                        [414934.0, 6829326.0],
                        [407934.0, 6831360.0],
                        [404267.0, 6828490.0],
                        [399495.0, 6830885.0],
                    ]
                ],
            },
        },
    },
    8: {
        "feature_a": copy.deepcopy(feature_dpt_population_a),
        "feature_b": copy.deepcopy(feature_dpt_geometry_only_a),
        "merge_metadata": {
            "name": "FRANCE_DPT_WITH_POPULATION",
            "fields": {
                "CODE_DEPT": {"type": "String", "width": 2, "index": 0},
                "NOM_DEPT": {"type": "String", "width": 23, "index": 1},
                "POPULATION": {"type": "Integer", "index": 2},
                "DENSITY": {"type": "Real", "width": 7, "precision": 2, "index": 3},
            },
            "geometry_ref": {"type": {"Point"}},
        },
        "field_name_filter_a": ["CODE_DEPT", "NOM_DEPT", "POPULATION", "DENSITY"],
        "field_name_filter_b": [],
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_b",
        "return_value": metadata_geometry_ref_type_not_match,
    },
    9: {
        "feature_a": copy.deepcopy(feature_dpt_geometry_only_a),
        "feature_b": copy.deepcopy(feature_dpt_data_only_a),
        "merge_metadata": None,
        "field_name_filter_a": None,
        "field_name_filter_b": None,
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_c",
        "return_value": geometry_ref_feature,
    },
    10: {
        "feature_a": copy.deepcopy(feature_dpt_population_a),
        "feature_b": copy.deepcopy(feature_dpt_data_and_geometry_a),
        "merge_metadata": metadata_fr_dept_population_geometry,
        "field_name_filter_a": ["POPULATION", "DENSITY"],
        "field_name_filter_b": ["CODE_DEPT", "NOM_DEPT"],
        "rename_fields_a": None,
        "rename_fields_b": None,
        "geometry_ref": "feature_b",
        "return_value": feature_dpt_population_geometry_a,
    },
}


def test_all():

    # merge metadata
    print(test_function(merge_metadata, merge_metadata_parameters))

    # merge feature
    print(test_function(merge_feature, merge_feature_parameters))


if __name__ == "__main__":
    test_all()



from geoformat.index.attributes.hash import create_attribute_index

from tests.data.geolayers import geolayer_fr_dept_population
from tests.data.index import (
    geolayer_fr_dept_population_CODE_DEPT_hash_index,
    geolayer_fr_dept_population_INSEE_REG_hash_index,
)

from tests.utils.tests_utils import test_function

create_attribute_index_parameters = {
    0: {
        "geolayer": geolayer_fr_dept_population,
        "field_name": "CODE_DEPT",
        "return_value": geolayer_fr_dept_population_CODE_DEPT_hash_index
    },
    1: {
        "geolayer": geolayer_fr_dept_population,
        "field_name": "INSEE_REG",
        "return_value": geolayer_fr_dept_population_INSEE_REG_hash_index
    }
}

def test_all():

    # create_attribute_index
    print(test_function(create_attribute_index, create_attribute_index_parameters))

if __name__ == '__main__':
    test_all()


import shutil
from tests.utils.tests_utils import test_function
from geoformat.driver.csv_driver import (
    csv_to_geolayer,
    geoformat_feature_to_csv_feature,
    feature_attributes_to_csv_attributes,
    geolayer_to_csv,
)
from pathlib import Path

from tests.data.geolayers import (
    geolayer_attributes_only,
    geolayer_attributes_only_without_none_value,
    geolayer_attributes_to_force_only_forced,
    geolayer_fr_dept_data_only,
    geolayer_fr_dept_geometry_only,
    geolayer_fr_dept_data_and_geometry,
    geolayer_paris_velib,
    geolayer_paris_velib_str,
    geolayer_paris_velib_str_no_header,
)
from tests.data.features import (
    feature_attributes_only,
    feature_dpt_geometry_only_a,
    feature_dpt_data_and_geometry_a,
)


# declare path
file_path_base = Path(__file__).parent.parent.parent.parent.joinpath

geolayer_attributes_only_path = file_path_base(
    "tests/geoformat/driver/test/attributes_only.csv"
)

geolayer_attributes_to_force_only_forced_path = file_path_base(
    "tests/geoformat/driver/test/attributes_to_force_only_forced.csv"
)
geolayer_fr_dept_data_only_path = file_path_base(
    "tests/geoformat/driver/test/fr_dept_data_only.csv"
)
geolayer_fr_dept_geometry_only_path = file_path_base(
    "tests/geoformat/driver/test/fr_dept_geometry_only.csv"
)
geolayer_fr_dept_data_and_geometry_path = file_path_base(
    "tests/geoformat/driver/test/fr_dept_data_and_geometry.csv"
)
geolayer_paris_velib_path = file_path_base(
    "tests/geoformat/driver/test/paris_velib.csv"
)
geolayer_paris_velib_str_path = file_path_base(
    "tests/geoformat/driver/test/paris_velib_str.csv"
)
paris_velib_str_no_header_path = file_path_base(
    "tests/geoformat/driver/test/paris_velib_str_no_header.csv"
)

geoformat_feature_to_csv_feature_parameters = {
    0: {
        "feature": feature_attributes_only,
        "recast_field_mapping": {
            "field_integer": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_integer_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_real": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_real_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_string_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_date": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_time": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_datetime": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_binary": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_boolean": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
        },
        "string_field_name": ["field_string"],
        "null_string": None,
        "quote_character": "",
        "write_geometry": False,
        "geometry_format": None,
        "return_value": {
            "attributes": {
                "field_integer": "586",
                "field_integer_list": "[5879, 8557]",
                "field_real": "8789.97568",
                "field_real_list": "[89798.3654, 8757.97568]",
                "field_string": "salut",
                "field_string_list": "['bonjour', 'monsieur']",
                "field_none": None,
                "field_date": "2020-03-31",
                "field_time": "11:22:10.000999",
                "field_datetime": "2020-03-31 11:22:10.000999",
                "field_binary": "00000000040000000200000000010000000000000000000000000000000000000000013ff00000000000003ff0000000000000",
                "field_boolean": "True",
            },
            "geometry": {},
        },
    },
    1: {
        "feature": feature_attributes_only,
        "recast_field_mapping": {
            "field_integer": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_integer_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_real": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_real_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_string_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_date": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_time": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_datetime": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_binary": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_boolean": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
        },
        "string_field_name": ["field_string"],
        "null_string": "",
        "quote_character": "",
        "write_geometry": False,
        "geometry_format": None,
        "return_value": {
            "attributes": {
                "field_integer": "586",
                "field_integer_list": "[5879, 8557]",
                "field_real": "8789.97568",
                "field_real_list": "[89798.3654, 8757.97568]",
                "field_string": "salut",
                "field_string_list": "['bonjour', 'monsieur']",
                "field_none": "",
                "field_date": "2020-03-31",
                "field_time": "11:22:10.000999",
                "field_datetime": "2020-03-31 11:22:10.000999",
                "field_binary": "00000000040000000200000000010000000000000000000000000000000000000000013ff00000000000003ff0000000000000",
                "field_boolean": "True",
            },
            "geometry": {},
        },
    },
    2: {
        "feature": feature_attributes_only,
        "recast_field_mapping": {
            "field_integer": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_integer_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_real": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_real_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_string_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_date": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_time": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_datetime": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_binary": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_boolean": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
        },
        "string_field_name": ["field_string"],
        "null_string": "",
        "quote_character": '"',
        "write_geometry": False,
        "geometry_format": None,
        "return_value": {
            "attributes": {
                "field_integer": "586",
                "field_integer_list": "[5879, 8557]",
                "field_real": "8789.97568",
                "field_real_list": "[89798.3654, 8757.97568]",
                "field_string": '"salut"',
                "field_string_list": "['bonjour', 'monsieur']",
                "field_none": "",
                "field_date": "2020-03-31",
                "field_time": "11:22:10.000999",
                "field_datetime": "2020-03-31 11:22:10.000999",
                "field_binary": "00000000040000000200000000010000000000000000000000000000000000000000013ff00000000000003ff0000000000000",
                "field_boolean": "True",
            },
            "geometry": {},
        },
    },
    3: {
        "feature": feature_dpt_geometry_only_a,
        "recast_field_mapping": {},
        "string_field_name": [],
        "null_string": "",
        "quote_character": '"',
        "write_geometry": True,
        "geometry_format": "WKT",
        "return_value": {
            "attributes": {},
            "geometry": "POLYGON ((399495.0 6830885.0,398130.0 6822559.0,400321.0 6810723.0,395852.0 6803336.0,398626.0 6784333.0,400465.0 6781914.0,400197.0 6773697.0,394099.0 6773357.0,390140.0 6770978.0,386941.0 6760260.0,382932.0 6754022.0,389872.0 6749698.0,393110.0 6750366.0,402067.0 6747685.0,404251.0 6751414.0,412442.0 6746090.0,419671.0 6744167.0,429458.0 6743442.0,440863.0 6746201.0,446732.0 6745443.0,446459.0 6750432.0,442128.0 6753611.0,448124.0 6758669.0,447308.0 6764356.0,455060.0 6767070.0,451057.0 6776681.0,459373.0 6778102.0,460615.0 6783387.0,458409.0 6789055.0,466280.0 6794064.0,465298.0 6799724.0,467628.0 6811401.0,473893.0 6813452.0,474394.0 6821359.0,467262.0 6822174.0,466087.0 6830999.0,463434.0 6833996.0,457920.0 6827997.0,451256.0 6826715.0,446687.0 6829012.0,441174.0 6828584.0,437568.0 6825109.0,429868.0 6822252.0,422197.0 6821752.0,414934.0 6829326.0,407934.0 6831360.0,404267.0 6828490.0,399495.0 6830885.0))",
        },
    },
    4: {
        "feature": feature_dpt_geometry_only_a,
        "recast_field_mapping": {},
        "string_field_name": [],
        "null_string": "",
        "quote_character": '"',
        "write_geometry": True,
        "geometry_format": "WKB",
        "return_value": {
            "attributes": {},
            "geometry": "000000000300000001000000304118621c00000000415a0ec94000000041184cc800000000415a06a7c000000041186f04000000004159fb18c000000041182930000000004159f3e20000000041185488000000004159e1534000000041187144000000004159def68000000041186d14000000004159d6f04000000041180dcc000000004159d69b400000004117cff0000000004159d4488000000041179df4000000004159c9d10000000041175f50000000004159c3b9800000004117cbc0000000004159bf80800000004117fe58000000004159c0278000000041188a4c000000004159bd89400000004118ac6c000000004159c12d8000000041192c68000000004159bbfa8000000041199d5c000000004159ba19c0000000411a3648000000004159b96480000000411ae87c000000004159bc1640000000411b4430000000004159bb58c0000000411b3fec000000004159c03800000000411afc40000000004159c352c0000000411b59f0000000004159c84340000000411b4d30000000004159cdd100000000411bc650000000004159d07780000000411b87c4000000004159d9da40000000411c09b4000000004159db3d80000000411c1d1c000000004159e066c0000000411bfaa4000000004159e5efc0000000411c75a0000000004159ead400000000411c6648000000004159f05b00000000411c8ab0000000004159fbc240000000411cec94000000004159fdc300000000411cf46800000000415a057bc0000000411c84f800000000415a064780000000411c729c00000000415a0ee5c0000000411c492800000000415a11d300000000411bf30000000000415a0bf740000000411b8ae000000000415a0ab6c0000000411b437c00000000415a0cf500000000411aed5800000000415a0c8a00000000411ab50000000000415a092540000000411a3cb000000000415a065b000000004119c4d400000000415a05de000000004119535800000000415a0d43800000004118e5f800000000415a0f40000000004118acac00000000415a0c72800000004118621c00000000415a0ec940000000",
        },
    },
    5: {
        "feature": feature_dpt_geometry_only_a,
        "recast_field_mapping": {},
        "string_field_name": [],
        "null_string": "",
        "quote_character": '"',
        "write_geometry": True,
        "geometry_format": "GEOJSON",
        "return_value": {
            "attributes": {},
            "geometry": '{"type": "Polygon", "coordinates": [[[399495.0, 6830885.0], [398130.0, 6822559.0], [400321.0, 6810723.0], [395852.0, 6803336.0], [398626.0, 6784333.0], [400465.0, 6781914.0], [400197.0, 6773697.0], [394099.0, 6773357.0], [390140.0, 6770978.0], [386941.0, 6760260.0], [382932.0, 6754022.0], [389872.0, 6749698.0], [393110.0, 6750366.0], [402067.0, 6747685.0], [404251.0, 6751414.0], [412442.0, 6746090.0], [419671.0, 6744167.0], [429458.0, 6743442.0], [440863.0, 6746201.0], [446732.0, 6745443.0], [446459.0, 6750432.0], [442128.0, 6753611.0], [448124.0, 6758669.0], [447308.0, 6764356.0], [455060.0, 6767070.0], [451057.0, 6776681.0], [459373.0, 6778102.0], [460615.0, 6783387.0], [458409.0, 6789055.0], [466280.0, 6794064.0], [465298.0, 6799724.0], [467628.0, 6811401.0], [473893.0, 6813452.0], [474394.0, 6821359.0], [467262.0, 6822174.0], [466087.0, 6830999.0], [463434.0, 6833996.0], [457920.0, 6827997.0], [451256.0, 6826715.0], [446687.0, 6829012.0], [441174.0, 6828584.0], [437568.0, 6825109.0], [429868.0, 6822252.0], [422197.0, 6821752.0], [414934.0, 6829326.0], [407934.0, 6831360.0], [404267.0, 6828490.0], [399495.0, 6830885.0]]]}',
        },
    },
    6: {
        "feature": feature_dpt_data_and_geometry_a,
        "recast_field_mapping": {},
        "string_field_name": ["CODE_DEPT", "NOM_DEPT"],
        "null_string": "",
        "quote_character": '"',
        "write_geometry": True,
        "geometry_format": "GEOJSON",
        "return_value": {
            "attributes": {"CODE_DEPT": '"53"', "NOM_DEPT": '"MAYENNE"'},
            "geometry": '{"type": "Polygon", "coordinates": [[[399495.0, 6830885.0], [398130.0, 6822559.0], [400321.0, 6810723.0], [395852.0, 6803336.0], [398626.0, 6784333.0], [400465.0, 6781914.0], [400197.0, 6773697.0], [394099.0, 6773357.0], [390140.0, 6770978.0], [386941.0, 6760260.0], [382932.0, 6754022.0], [389872.0, 6749698.0], [393110.0, 6750366.0], [402067.0, 6747685.0], [404251.0, 6751414.0], [412442.0, 6746090.0], [419671.0, 6744167.0], [429458.0, 6743442.0], [440863.0, 6746201.0], [446732.0, 6745443.0], [446459.0, 6750432.0], [442128.0, 6753611.0], [448124.0, 6758669.0], [447308.0, 6764356.0], [455060.0, 6767070.0], [451057.0, 6776681.0], [459373.0, 6778102.0], [460615.0, 6783387.0], [458409.0, 6789055.0], [466280.0, 6794064.0], [465298.0, 6799724.0], [467628.0, 6811401.0], [473893.0, 6813452.0], [474394.0, 6821359.0], [467262.0, 6822174.0], [466087.0, 6830999.0], [463434.0, 6833996.0], [457920.0, 6827997.0], [451256.0, 6826715.0], [446687.0, 6829012.0], [441174.0, 6828584.0], [437568.0, 6825109.0], [429868.0, 6822252.0], [422197.0, 6821752.0], [414934.0, 6829326.0], [407934.0, 6831360.0], [404267.0, 6828490.0], [399495.0, 6830885.0]]]}',
        },
    },
}

feature_attributes_to_csv_attributes_parameters = {
    0: {
        "attributes": feature_attributes_only["attributes"],
        "recast_field_mapping": {
            "field_integer": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_integer_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_real": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_real_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_string_list": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_date": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_time": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_datetime": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_binary": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
            "field_boolean": {
                "recast_value_to_python_type": str,
                "resize_value_width": None,
                "resize_value_precision": None,
            },
        },
        "return_value": {
            "field_integer": "586",
            "field_integer_list": "[5879, 8557]",
            "field_real": "8789.97568",
            "field_real_list": "[89798.3654, 8757.97568]",
            "field_string": "salut",
            "field_string_list": "['bonjour', 'monsieur']",
            "field_none": None,
            "field_date": "2020-03-31",
            "field_time": "11:22:10.000999",
            "field_datetime": "2020-03-31 11:22:10.000999",
            "field_binary": "00000000040000000200000000010000000000000000000000000000000000000000013ff00000000000003ff0000000000000",
            "field_boolean": "True",
        },
    },
    1: {"attributes": {}, "recast_field_mapping": {}, "return_value": {}},
}

geolayer_to_csv_parameters = {
    0: {
        "geolayer": geolayer_attributes_only,
        "path": geolayer_attributes_only_path,
        "overwrite": True,
        "add_extension": False,
        "delimiter": ";",  # we prefer semicolon because StringList, IntegerList and FloatList
        "header": True,
        "null_string": "",
        "quote_character": "",
        "write_geometry": True,
        "geometry_format": "WKT",
        "geometry_field_name": "geom",
        "encoding": "utf8",
        "return_value": geolayer_attributes_only_path,
    },
    1: {
        "geolayer": geolayer_attributes_to_force_only_forced,
        "path": geolayer_attributes_to_force_only_forced_path,
        "overwrite": True,
        "add_extension": False,
        "delimiter": ";",  # we prefer semicolon because StringList, IntegerList and FloatList
        "header": True,
        "null_string": "",
        "quote_character": "",
        "write_geometry": True,
        "geometry_format": "WKT",
        "geometry_field_name": "geom",
        "encoding": "utf8",
        "return_value": geolayer_attributes_to_force_only_forced_path,
    },
    2: {
        "geolayer": geolayer_fr_dept_data_only,
        "path": geolayer_fr_dept_data_only_path,
        "overwrite": True,
        "add_extension": False,
        "delimiter": ";",  # we prefer semicolon because StringList, IntegerList and FloatList
        "header": True,
        "null_string": "",
        "quote_character": "",
        "write_geometry": True,
        "geometry_format": "WKT",
        "geometry_field_name": "geom",
        "encoding": "utf8",
        "return_value": geolayer_fr_dept_data_only_path,
    },
    3: {
        "geolayer": geolayer_fr_dept_geometry_only,
        "path": geolayer_fr_dept_geometry_only_path,
        "overwrite": True,
        "add_extension": False,
        "delimiter": ";",  # we prefer semicolon because StringList, IntegerList and FloatList
        "header": True,
        "null_string": "",
        "quote_character": "",
        "write_geometry": True,
        "geometry_format": "WKT",
        "geometry_field_name": "geom",
        "encoding": "utf8",
        "return_value": geolayer_fr_dept_geometry_only_path,
    },
    4: {
        "geolayer": geolayer_fr_dept_data_and_geometry,
        "path": geolayer_fr_dept_data_and_geometry_path,
        "overwrite": True,
        "add_extension": False,
        "delimiter": ";",  # we prefer semicolon because StringList, IntegerList and FloatList
        "header": True,
        "null_string": "",
        "quote_character": "",
        "write_geometry": True,
        "geometry_format": "WKT",
        "geometry_field_name": "geom",
        "encoding": "utf8",
        "return_value": geolayer_fr_dept_data_and_geometry_path,
    },
    5: {
        "geolayer": geolayer_paris_velib,
        "path": geolayer_paris_velib_path,
        "overwrite": True,
        "add_extension": False,
        "delimiter": ";",  # we prefer semicolon because StringList, IntegerList and FloatList
        "header": True,
        "null_string": "",
        "quote_character": "",
        "write_geometry": True,
        "geometry_format": "WKT",
        "geometry_field_name": "geom",
        "encoding": "utf8",
        "return_value": geolayer_paris_velib_path,
    },
    6: {
        "geolayer": geolayer_paris_velib_str,
        "path": geolayer_paris_velib_str_path,
        "overwrite": True,
        "add_extension": False,
        "delimiter": ";",  # we prefer semicolon because StringList, IntegerList and FloatList
        "header": True,
        "null_string": "",
        "quote_character": '"',
        "write_geometry": True,
        "geometry_format": "WKT",
        "geometry_field_name": "geom",
        "encoding": "utf8",
        "return_value": geolayer_paris_velib_str_path,
    },
    7: {
        "geolayer": geolayer_paris_velib_str,
        "path": paris_velib_str_no_header_path,
        "overwrite": True,
        "add_extension": False,
        "delimiter": ";",  # we prefer semicolon because StringList, IntegerList and FloatList
        "header": False,
        "null_string": "",
        "quote_character": "",
        "write_geometry": True,
        "geometry_format": "WKT",
        "geometry_field_name": "geom",
        "encoding": "utf8",
        "return_value": paris_velib_str_no_header_path,
    },
}

csv_to_geolayer_parameters = {
    0: {
        "path": geolayer_attributes_only_path,
        "geolayer_name": None,
        "delimiter": ";",
        "header": True,
        "null_string": "",
        "field_name_filter": None,
        "force_field_conversion": True,
        "serialize": False,
        "geometry_field": False,
        "geometry_field_name": None,
        "geometry_format": "WKT",
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_attributes_only_without_none_value,
    },
    1: {
        "path": geolayer_attributes_to_force_only_forced_path,
        "geolayer_name": "geolayer_attributes_to_force_only",
        "delimiter": ";",
        "header": True,
        "null_string": "",
        "field_name_filter": None,
        "force_field_conversion": True,
        "serialize": False,
        "geometry_field": False,
        "geometry_field_name": None,
        "geometry_format": "WKT",
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_attributes_to_force_only_forced,
    },
    2: {
        "path": geolayer_fr_dept_data_only_path,
        "geolayer_name": "FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_ONLY",
        "delimiter": ";",
        "header": True,
        "field_name_filter": None,
        "force_field_conversion": True,
        "serialize": False,
        "geometry_field": False,
        "geometry_field_name": None,
        "geometry_format": "WKT",
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_fr_dept_data_only,
    },
    3: {
        "path": geolayer_fr_dept_geometry_only_path,
        "geolayer_name": "FRANCE_DPT_GENERALIZE_LAMB93_ROUND_GEOMETRY_ONLY",
        "delimiter": ";",
        "header": True,
        "field_name_filter": None,
        "force_field_conversion": True,
        "serialize": False,
        "geometry_field": True,
        "geometry_field_name": "geom",
        "geometry_format": "WKT",
        "crs": 2154,
        "bbox_extent": True,
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_fr_dept_geometry_only,
    },
    4: {
        "path": geolayer_fr_dept_data_and_geometry_path,
        "geolayer_name": "FRANCE_DPT_GENERALIZE_LAMB93_ROUND_DATA_AND_GEOMETRY",
        "delimiter": ";",
        "header": True,
        "field_name_filter": None,
        "force_field_conversion": True,
        "serialize": False,
        "geometry_field": True,
        "geometry_field_name": "geom",
        "geometry_format": "WKT",
        "crs": 2154,
        "bbox_extent": False,
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_fr_dept_data_and_geometry,
    },
    5: {
        "path": geolayer_paris_velib_path,
        "geolayer_name": "geolayer_paris_velib",
        "delimiter": ";",
        "header": True,
        "field_name_filter": None,
        "force_field_conversion": True,
        "serialize": False,
        "geometry_field": False,
        "geometry_field_name": None,
        "geometry_format": "WKT",
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_paris_velib,
    },
    6: {
        "path": geolayer_paris_velib_str_path,
        "delimiter": ";",
        "header": True,
        "field_name_filter": None,
        "force_field_conversion": False,
        "serialize": False,
        "geometry_field": False,
        "geometry_field_name": None,
        "geometry_format": "WKT",
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_paris_velib_str,
    },
    7: {
        "path": paris_velib_str_no_header_path,
        "geolayer_name": "paris_velib_str",
        "delimiter": ";",
        "header": False,
        "field_name_filter": None,
        "force_field_conversion": False,
        "serialize": False,
        "geometry_field": False,
        "geometry_field_name": None,
        "geometry_format": "WKT",
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_paris_velib_str_no_header,
    },
    8: {
        "path": "https://framagit.org/Guilhain/Geoformat/-/raw/master/data/csv/velib-disponibilite-en-temps-reel.csv?inline=false",
        "geolayer_name": "geolayer_paris_velib",
        "delimiter": ";",
        "header": True,
        "field_name_filter": None,
        "force_field_conversion": True,
        "serialize": False,
        "geometry_field": False,
        "geometry_field_name": None,
        "geometry_format": "WKT",
        "encoding": "utf8",
        "http_headers": None,
        "return_value": geolayer_paris_velib,
    },
}


def test_all():
    # create test dir
    dir_test_path = file_path_base("tests/geoformat/driver/test/")
    if not dir_test_path.exists():
        dir_test_path.mkdir()

    # geoformat_feature_to_csv_feature
    print(
        test_function(
            geoformat_feature_to_csv_feature,
            geoformat_feature_to_csv_feature_parameters,
        )
    )

    # feature_attributes_to_csv_attributes
    print(
        test_function(
            feature_attributes_to_csv_attributes,
            feature_attributes_to_csv_attributes_parameters,
        )
    )

    # geolayer_to_csv
    print(test_function(geolayer_to_csv, geolayer_to_csv_parameters))

    # csv_to_geolayer
    print(test_function(csv_to_geolayer, csv_to_geolayer_parameters))

    shutil.rmtree(path=dir_test_path)


if __name__ == "__main__":
    test_all()



from geoformat.geoprocessing.area import (
    geometry_area
)
from tests.data.geometries import (
    POINT,
    POINT_EMPTY,
    LINESTRING,
    LINESTRING_EMPTY,
    POLYGON,
    POLYGON_EMPTY,
    MULTIPOINT,
    MULTIPOINT_EMPTY,
    MULTILINESTRING,
    MULTILINESTRING_EMPTY,
    MULTIPOLYGON,
    MULTIPOLYGON_EMPTY,
    GEOMETRYCOLLECTION,
    GEOMETRYCOLLECTION_EMPTY
)
from tests.utils.tests_utils import test_function

geometry_area_parameters = {
    0: {
        "geometry": POINT,
        "return_value": 0,
    },
    1: {
        "geometry": POINT_EMPTY,
        "return_value": 0,
    },
    2: {
        "geometry": LINESTRING,
        "return_value": 0
    },
    3: {
        "geometry": LINESTRING_EMPTY,
        "return_value": 0
    },
    4: {
        "geometry": POLYGON,
        "return_value": 4675.238814000001
    },
    5: {
        "geometry": POLYGON_EMPTY,
        "return_value": 0,
    },
    6: {
        "geometry": MULTIPOINT,
        "return_value": 0
    },
    7: {
        "geometry": MULTIPOINT_EMPTY,
        "return_value": 0
    },
    8: {
        "geometry": MULTILINESTRING,
        "return_value": 0
    },
    9: {
        "geometry": MULTILINESTRING_EMPTY,
        "return_value": 0
    },
    10: {
        "geometry": MULTIPOLYGON,
        "return_value": 5198.540129999999
    },
    11: {
        "geometry": MULTIPOLYGON_EMPTY,
        "return_value": 0
    },
    12: {
        "geometry": GEOMETRYCOLLECTION,
        "return_value": 9873.778944000002
    },
    13: {
        "geometry": GEOMETRYCOLLECTION_EMPTY,
        "return_value": 0
    },
}


def test_all():
    # geometry_area
    print(test_function(geometry_area, geometry_area_parameters))


if __name__ == '__main__':
    test_all()




from geoformat.processing.data.field_statistics import field_statistics
from tests.utils.tests_utils import test_function
from tests.data.geolayers import (
    geolayer_fr_dept_population,
    geolayer_btc_price_sample,
)

field_statistics_parameters = {
    0: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'SUM']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'SUM': 64638088}}
    },
    2: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'MEAN']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'MEAN': 673313.4166666665}}
    },
    3: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'MIN']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'MIN': 76601}}
    },
    4: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'MAX']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'MAX': 2604361}}
    },
    5: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'RANGE']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'RANGE': 2527760}}
    },
    6: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'STD']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'STD': 515274.66505417065}}
    },
    7: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'COUNT']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'COUNT': 96}}
    },
    8: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'FIRST']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'FIRST': 191091}}
    },
    9: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'LAST']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'LAST': 2187526}}
    },
    10: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'VARIANCE']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'VARIANCE': 265507980446.68777}}
    },
    11: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [[('POPULATION', 'AREA'), 'WEIGHTED_MEAN']],
        "bessel_correction": True,
        "return_value": {('POPULATION', 'AREA'): {'WEIGHTED_MEAN': 622009.6380951514}}
    },
    12: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [[('POPULATION', 'AREA'), 'COVARIANCE']],
        "bessel_correction": True,
        "return_value": {('POPULATION', 'AREA'): {'COVARIANCE': -293045606.6781337}}
    },
    13: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'SUM']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'SUM': 64638088}}
    },
    14: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'MEAN']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'MEAN': 673313.4166666665}}
    },
    15: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'MIN']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'MIN': 76601}}
    },
    16: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'MAX']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'MAX': 2604361}}
    },
    17: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'RANGE']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'RANGE': 2527760}}
    },
    18: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'STD']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'STD': 512583.9173413801}}
    },
    19: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'COUNT']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'COUNT': 96}}
    },
    20: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'FIRST']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'FIRST': 191091}}
    },
    21: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'LAST']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'LAST': 2187526}}
    },
    22: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'VARIANCE']],
        "bessel_correction": False,
        "return_value": {'POPULATION': {'VARIANCE': 262742272317.0348}}
    },
    23: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [[('POPULATION', 'AREA'), 'WEIGHTED_MEAN']],
        "bessel_correction": False,
        "return_value": {('POPULATION', 'AREA'): {'WEIGHTED_MEAN': 622009.6380951514}}
    },
    24: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [[('POPULATION', 'AREA'), 'COVARIANCE']],
        "bessel_correction": False,
        "return_value": {('POPULATION', 'AREA'): {'COVARIANCE': -293045606.6781337}}
    },
    25: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'ALL']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'COUNT': 96, 'SUM': 64638088, 'RANGE': 2527760, 'MIN': 76601, 'MEAN': 673313.4166666665, 'MAX': 2604361, 'STD': 515274.66505417065, 'FIRST': 191091, 'LAST': 2187526, 'VARIANCE': 265507980446.68777}}
    },
    26: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [['POPULATION', 'ALL'], ['AREA', 'ALL']],
        "bessel_correction": True,
        "return_value": {'POPULATION': {'COUNT': 96, 'SUM': 64638088, 'RANGE': 2604185.37, 'MIN': 76601, 'MEAN': 673313.4166666665, 'MAX': 2604361, 'STD': 515274.66505417065, 'FIRST': 191091, 'LAST': 2187526, 'VARIANCE': 265507980446.68777}, 'AREA': {'COUNT': 96, 'SUM': 548349.0499999999, 'RANGE': 2604255.56, 'MIN': 105.44, 'MEAN': 5711.969270833332, 'MAX': 10068.74, 'STD': 1942.902658309954, 'FIRST': 6304.33, 'LAST': 105.44, 'VARIANCE': 3774870.7396678855}}
    },
    27: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [[('POPULATION', 'AREA'), 'ALL']],
        "bessel_correction": True,
        "return_value": {('POPULATION', 'AREA'): {'WEIGHTED_MEAN': 622009.6380951514, 'COVARIANCE': -293045606.6781337}}
    },
    28: {
        "geolayer": geolayer_fr_dept_population,
        "statistic_field": [[('POPULATION', 'AREA'), 'ALL'], ['POPULATION', 'ALL']],
        "bessel_correction": True,
        "return_value": {('POPULATION', 'AREA'): {'WEIGHTED_MEAN': 622009.6380951514, 'COVARIANCE': -293045606.6781337}, 'POPULATION': {'COUNT': 96, 'SUM': 64638088, 'RANGE': 2527760, 'MIN': 76601, 'MEAN': 673313.4166666665, 'MAX': 2604361, 'STD': 515274.66505417065, 'FIRST': 191091, 'LAST': 2187526, 'VARIANCE': 265507980446.68777}}
    },
    29: {
        "geolayer": geolayer_btc_price_sample,
        "statistic_field": [["USD_PRICE_CLOSE", "MEAN"]],
        "bessel_correction": True,
        "return_value": {"USD_PRICE_CLOSE": {'MEAN': 32779.41181818183}},
    }
}

def test_all():

    # field_statistics
    print(test_function(field_statistics, field_statistics_parameters))


if __name__ == '__main__':
    test_all()